"""
Read graphs in LEDA format.
See http://www.algorithmic-solutions.info/leda_guide/graphs/leda_native_graph_fileformat.html

"""
# Original author: D. Eppstein, UC Irvine, August 12, 2003.
# The original code at http://www.ics.uci.edu/~eppstein/PADS/ is public domain.
__author__ = """Aric Hagberg (hagberg@lanl.gov)"""
__date__ = """"""
__credits__ = """"""
__revision__ = ""
#    Copyright (C) 2004-2007 by 
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Pieter Swart <swart@lanl.gov>
#    Distributed under the terms of the GNU Lesser General Public License
#    http://www.gnu.org/copyleft/lesser.html

import networkx
from networkx.exception import NetworkXException, NetworkXError
from networkx.utils import _get_fh, is_string_like
	
def read_leda(path):
    """Read graph in GraphML format from path.
    Returns an XGraph or XDiGraph."""
    fh=_get_fh(path,mode='r')        
    G=parse_leda(fh)
    return G
	
def parse_leda(lines):
    """Parse LEDA.GRAPH format from string or iterable.
    Returns an XGraph or XDiGraph."""
    if is_string_like(lines): lines=iter(lines.split('\n'))
    lines = iter([line.rstrip('\n') for line in lines \
            if not (line.startswith('#') or line.startswith('\n') or line=='')])
    for i in range(3):
        lines.next()
    # Graph
    du = int(lines.next())	# -1 directed, -2 undirected
    if du==-1:
        G = networkx.XDiGraph()
    else:
        G = networkx.XGraph()
        
    # Nodes
    n =int(lines.next())	# number of vertices
    node={}
    for i in range(1,n+1):  # LEDA counts from 1 to n
        symbol=lines.next()[2:-3]  # strip out data from |{data}|
        if symbol=="": symbol=str(i) # use int if no label - could be trouble
        node[i]=symbol

    G.add_nodes_from([s for i,s in node.items()])
	
    # Edges
    m = int(lines.next())	# number of edges
    for i in range(m):
        try:
            s,t,reversal,label=lines.next().split()
        except:
            raise NetworkXError,\
                  'Too few fields in LEDA.GRAPH edge %d' % (i+1)
        # BEWARE: no handling of reversal edges
        G.add_edge(node[int(s)],node[int(t)],label[2:-2])
    return G


def _test_suite():
    import doctest
    suite = doctest.DocFileSuite('tests/readwrite/leda.txt',
                                 package='networkx')
    return suite

if __name__ == "__main__":
    import os 
    import sys
    import unittest
    if sys.version_info[:2] < (2, 4):
        print "Python version 2.4 or later required for tests (%d.%d detected)." %  sys.version_info[:2]
        sys.exit(-1)
    # directory of networkx package (relative to this)
    nxbase=sys.path[0]+os.sep+os.pardir
    sys.path.insert(0,nxbase) # prepend to search path
    unittest.TextTestRunner().run(_test_suite())



# -*- coding: utf-8 -*-
"""
Algorithms for directed acyclic graphs (DAGs).
"""
__author__ = """Aric Hagberg (hagberg@lanl.gov)\nDan Schult(dschult@colgate.edu)"""
___revision__ = ""
#    Copyright (C) 2006 by 
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Pieter Swart <swart@lanl.gov>
#    Distributed under the terms of the GNU Lesser General Public License
#    http://www.gnu.org/copyleft/lesser.html
import networkx

def is_directed_acyclic_graph(G):
    """Return True if the graph G is a directed acyclic graph (DAG).

    Otherwise return False.
    
    """
    if topological_sort(G) is None:
        return False
    else:
        return True

def topological_sort(G):
    """
    Return a list of nodes of the digraph G in topological sort order.

    A topological sort is a nonunique permutation of the nodes
    such that an edge from u to v implies that u appears before v in the
    topological sort order.

    If G is not a directed acyclic graph no topological sort exists
    and the Python keyword None is returned.

    This algorithm is based on a description and proof at
    http://www2.toki.or.id/book/AlgDesignManual/book/book2/node70.htm

    See also is_directed_acyclic_graph()
    
    """
    # nonrecursive version

    seen={}
    order_explored=[] # provide order and 
    explored={}       # fast search without more general priorityDictionary
                     
    if not G.is_directed():  G.successors_iter=G.neighbors_iter

    for v in G.nodes_iter():     # process all vertices in G
        if v in explored: continue

        fringe=[v]   # nodes yet to look at
        while fringe:
            w=fringe[-1]  # depth first search
            if w in explored: # already looked down this branch
                fringe.pop()
                continue
            seen[w]=1     # mark as seen
            # Check successors for cycles and for new nodes
            new_nodes=[]
            for n in G.successors_iter(w):  
                if n not in explored:
                    if n in seen: return #CYCLE !!
                    new_nodes.append(n)
            if new_nodes:   # Add new_nodes to fringe
                fringe.extend(new_nodes)
            else:           # No new nodes so w is fully explored
                explored[w]=1
                order_explored.insert(0,w) # reverse order explored
                fringe.pop()    # done considering this node
    return order_explored

def topological_sort_recursive(G):
    """
    Return a list of nodes of the digraph G in topological sort order.

    This is a recursive version of topological sort.
    
    """
    # function for recursive dfs
    def _dfs(G,seen,explored,v):
        seen[v]=1
        for w in G.successors(v):
            if w not in seen: 
                if not _dfs(G,seen,explored,w):
                    return
            elif w in seen and w not in explored:
                # cycle Found--- no topological sort
                return False
        explored.insert(0,v) # inverse order of when explored 
        return v

    seen={}
    explored=[]

    if not G.is_directed():  G.successors=G.neighbors
    
    for v in G.nodes_iter():  # process all nodes
        if v not in explored:
            if not _dfs(G,seen,explored,v): 
                return 
    return explored


def _test_suite():
    import doctest
    suite = doctest.DocFileSuite('tests/dag.txt',package='networkx')
    return suite


if __name__ == "__main__":
    import os
    import sys
    import unittest
    if sys.version_info[:2] < (2, 4):
        print "Python version 2.4 or later required for tests (%d.%d detected)." %  sys.version_info[:2]
        sys.exit(-1)
    # directory of networkx package (relative to this)
    nxbase=sys.path[0]+os.sep+os.pardir
    sys.path.insert(0,nxbase) # prepend to search path
    unittest.TextTestRunner().run(_test_suite())
    






#!/usr/bin/env python
"""
This example illustrates the sudden appearance of a 
giant connected component in a binomial random graph.

Requires graphviz and pylab to draw.

"""
#    Copyright (C) 2006 by 
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Pieter Swart <swart@lanl.gov>
#    Distributed under the terms of the GNU Lesser General Public License
#    http://www.gnu.org/copyleft/lesser.html

from networkx import *
try:
    import pylab as P
except:
    print "pylab not found: see https://networkx.lanl.gov/Drawing.html for info"
    raise 

from math import *

n=150  # 150 nodes
p_giant=1.0/(n-1)      # p value at which giant component 
                       # (of size log(n) nodes) is expected
p_conn=log(n)/float(n) # p value at which graph is expected
                       # to become completely connected
# the next range of p values should be close to the above threshold
pvals=[0.003, 0.006, 0.008, 0.015] 

region=220 # for pylab 2x2 subplot layout
try: # drawing
     for p in pvals:    
	 G=binomial_graph(n,p)
	 pos=graphviz_layout(G)
	 region+=1
	 P.subplot(region)
         P.title("p = %6.3f"%(p))
	 draw(G,pos,
	      with_labels=False,
	      node_size=10
              )
         # identify largest connected component
         Gcc=connected_component_subgraphs(G)
         G0=Gcc[0] 
         draw_networkx_edges(G0,pos,
                             with_labels=False,
                             edge_color='r',
                             width=6.0
                             )
         # show other connected components
         for Gi in Gcc[1:]:
            if len(Gi)>1:
               draw_networkx_edges(Gi,pos,
                                   with_labels=False,
                                   edge_color='r',
                                   alpha=0.3,
                                   width=5.0
                                   )         
     P.savefig("binomial_giant.png")
     P.show() # display
except:
	print "Unable to draw: cannot find graphviz or pylab"

 


"""
Read and write NetworkX graphs.

Note that NetworkX graphs can contain any hashable Python object as
node (not just integers and strings).  So writing a NetworkX graph
as a text file may not always be what you want: see write_gpickle
and gread_gpickle for that case.

This module provides the following :

Edgelist format:
Useful for connected graphs with or without edge data.

   write_edgelist(G, path)
   G=read_edgelist(path)


"""
__author__ = """Aric Hagberg (hagberg@lanl.gov)\nDan Schult (dschult@colgate.edu)"""
__date__ = """"""
__credits__ = """"""
__revision__ = ""
#    Copyright (C) 2004-2007 by 
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Pieter Swart <swart@lanl.gov>
#    Distributed under the terms of the GNU Lesser General Public License
#    http://www.gnu.org/copyleft/lesser.html

import codecs
import locale
import string
import sys
import time

from networkx.utils import is_string_like,_get_fh
import networkx


def write_edgelist(G, path, comments="#", delimiter=' '):
    """Write graph G in edgelist format on file path.

    See read_edgelist for file format details.

    >>> write_edgelist(G, "file.edgelist")

    path can be a filehandle or a string with the name of the file.

    >>> fh=open("file.edgelist")
    >>> write_edgelist(G,fh)

    Filenames ending in .gz or .bz2 will be compressed.

    >>> write_edgelist(G, "file.edgelist.gz")

    The file will use the default text encoding on your system.
    It is possible to write files in other encodings by opening
    the file with the codecs module.  See doc/examples/unicode.py
    for hints.

    >>> import codecs
    >>> fh=codecs.open("file.edgelist",encoding='utf=8') # use utf-8 encoding
    >>> write_edgelist(G,fh)


    """
    fh=_get_fh(path,mode='w')

    pargs=comments+" "+string.join(sys.argv,' ')
    fh.write("%s\n" % (pargs))
    fh.write(comments+" GMT %s\n" % (time.asctime(time.gmtime())))
    fh.write(comments+" %s\n" % (G.name))
    for e in G.edges():
        for n in e:  # handle Graph or XGraph, two- or three-tuple
            if n is None: continue # don't write data for XGraph None 
            if is_string_like(n):
                fh.write(n+delimiter)
            else:
                fh.write(str(n)+delimiter)                         
        fh.write("\n")                     

def read_edgelist(path, comments="#", delimiter=' ',
                  create_using=None, nodetype=None, edgetype=None):
    """Read graph in edgelist format from path.

    >>> G=read_edgelist("file.edgelist")

    path can be a filehandle or a string with the name of the file.

    >>> fh=open("file.edgelist")
    >>> G=read_edgelist(fh)

    Filenames ending in .gz or .bz2 will be compressed.

    >>> G=read_edgelist("file.edgelist.gz")

    nodetype is an optional function to convert node strings to nodetype

    For example

    >>> G=read_edgelist("file.edgelist", nodetype=int)

    will attempt to convert all nodes to integer type

    Since nodes must be hashable, the function nodetype must return hashable
    types (e.g. int, float, str, frozenset - or tuples of those, etc.) 

    create_using is an optional networkx graph type, the default is
    Graph(), a simple undirected graph 

    >>> G=read_edgelist("file.edgelist",create_using=DiGraph())


    The comments character (default='#') at the beginning of a
    line indicates a comment line.

    The entries are separated by delimiter (default=' ').
    If whitespace is significant in node or edge labels you should use
    some other delimiter such as a tab or other symbol.  

    Example edgelist file format:: 

     # source target
     a b
     a c
     d e

    or for an XGraph() with edge data 

     # source target data
     a b 1
     a c 3.14159
     d e apple

    """
    if create_using is None:
        G=networkx.Graph()
    else:
        try:
            G=create_using
            G.clear()
        except:
            raise TypeError("Input graph is not a networkx graph type")

    # is this a XGraph or XDiGraph?
    if hasattr(G,'allow_multiedges')==True:
        xgraph=True
    else:
        xgraph=False

    fh=_get_fh(path)

    for line in fh.readlines():
        line = line[:line.find(comments)].strip()
        if not len(line): continue
#        if line.startswith("#") or line.startswith("\n"):
#            continue
#        line=line.strip() #remove trailing \n 
        # split line, should have 2 or three items
        s=line.split(delimiter)
        if len(s)==2:
            (u,v)=s
            d=None
        elif len(s)==3:
            (u,v,d)=s
        else:
            raise TypeError("Failed to read line: %s"%line)

        # convert types
        try:
            (u,v)=map(nodetype,(u,v))
        except:
            raise TypeError("Failed to convert edge (%s, %s) to type %s"\
                  %(u,v,nodetype))
        if d is not None and edgetype is not None:

            try:
               d=edgetype(d)
            except:
                raise TypeError("Failed to convert edge data (%s) to type %s"\
                                %(d, edgetype))
            
        if xgraph:
            G.add_edge(u,v,d)  # XGraph or XDiGraph
        else:
            G.add_edge(u,v)    # Graph or DiGraph
            
    return G



def _test_suite():
    import doctest
    suite = doctest.DocFileSuite('tests/readwrite/edgelist.txt',package='networkx')
    return suite


if __name__ == "__main__":
    import os 
    import sys
    import unittest
    if sys.version_info[:2] < (2, 4):
        print "Python version 2.4 or later required for tests (%d.%d detected)." %  sys.version_info[:2]
        sys.exit(-1)
    # directory of networkx package (relative to this)
    nxbase=sys.path[0]+os.sep+os.pardir
    sys.path.insert(0,nxbase) # prepend to search path
    unittest.TextTestRunner().run(_test_suite())
    



"""
Import and export networkx networks to dot format using pydot.

Provides:

 - write_dot()
 - read_dot()
 - graphviz_layout()
 - pydot_layout()

 - pydot_from_networkx()
 - networkx_from_pydot()

Either this module or nx_pygraphviz can be used to interface with graphviz.  

References:
 - pydot Homepage: http://www.dkbza.org/pydot.html
 - Graphviz:	   http://www.research.att.com/sw/tools/graphviz/
 - DOT Language:   http://www.research.att.com/~erg/graphviz/info/lang.html

"""
__author__ = """Aric Hagberg (hagberg@lanl.gov)"""
__date__ = "$Date: 2005-06-15 08:55:33 -0600 (Wed, 15 Jun 2005) $"
__credits__ = """"""
__revision__ = "$Revision: 1034 $"
#    Copyright (C) 2004,2005 by 
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Pieter Swart <swart@lanl.gov>
#    Distributed under the terms of the GNU Lesser General Public License
#    http://www.gnu.org/copyleft/lesser.html
import sys
try:
    import pydot
except ImportError:
#    print "Import Error: not able to import Python module pydot."
    raise


def write_dot(G,path=False):
    """Write G to a graphviz dot file."""
    P=pydot_from_networkx(G)
    if path:
        P.write_dot(path)
    else:
        print P.to_string() # write on stdout
    return

def read_dot(path=False):
    """Creates an networkx graph from a dot file"""

    if path:
        try: 
            fh=open(path,'r')
        except IOError:                     
            print "The file %s does not exist"%path
            raise
    else:
        fh=sys.stdin  # no path, read from stdin

    data=fh.read()        
    P=pydot.graph_from_dot_data(data)
    return networkx_from_pydot(P)


def pydot_from_networkx(N):
	"""Creates a pydot graph from an networkx graph N"""
	# node_orig = 1

        if N.is_directed():

		graph = pydot.Dot(graph_type='digraph')
	else:
		graph = pydot.Dot(graph_type='graph')

        for v in N.nodes_iter():
            graph.add_node(pydot.Node(str(v)))
        for e in N.edges_iter():
            if len(e)==2:
                (u,v)=e
                graph.add_edge(pydot.Edge(str(u),str(v)))
            if len(e)==3:
                (u,v,x)=e
                graph.add_edge(pydot.Edge(str(u),str(v),label=str(x)))

        # FIXME - add properties from networkx to pydot graph?
        # a nice feature would be to add node positions to pydot graph.
        return graph

def networkx_from_pydot(D, create_using=None):
	"""Creates an networkx graph from an pydot graph D"""
        import networkx

        if create_using is None:
            if D.get_type()=="digraph":
                create_using=networkx.DiGraph()
            else:
                create_using=networkx.Graph()

        N=networkx.empty_graph(0,create_using)

        N.name="%s"%(D.graph_name)

        for p in D.node_list:
            N.add_node(p.name)

        edges_seen = {}
        for e in D.edge_list:
            if e in edges_seen:
                continue
            N.add_edge(e.src,e.dst)
            edges_seen[e]=1

        # FIXME - add properties from pydot to networkx?
	return N

def graphviz_layout(G,prog='neato',root=None, **kwds):
    """Create layout using pydot and graphviz.
    Returns a dictionary of positions keyed by node.

    >>> pos=graphviz_layout(G)
    >>> pos=graphviz_layout(G,prog='dot')

    This is a wrapper for pydot_layout.

    """
    return pydot_layout(G=G,prog=prog,root=root,**kwds)


def pydot_layout(G,prog='neato',root=None, **kwds):
    """
    Create layout using pydot and graphviz.
    Returns a dictionary of positions keyed by node.

    >>> pos=pydot_layout(G)
    >>> pos=pydot_layout(G,prog='dot')
    
    """
    from networkx.drawing.nx_pydot import pydot_from_networkx
    try:
        import pydot
    except:
        print "Import Error: not able to import pydot."
        raise
    P=pydot_from_networkx(G)
    if root is not None :
        P.set("root",str(root))

    D=P.create_dot(prog=prog)

    if D=="":  # no data returned
        print "Graphviz layout with %s failed"%(prog)
        print
        print "To debug what happened try:"
        print "P=pydot_from_networkx(G)"
        print "P.write_dot(\"file.dot\")"
        print "And then run %s on file.dot"%(prog)
        return

    Q=pydot.graph_from_dot_data(D)

    node_pos={}
    for n in G.nodes():
        node=Q.get_node(str(n))
        if node.pos != None:
            xx,yy=node.pos.split(",")
            node_pos[n]=(float(xx),float(yy))
    return node_pos

def _test_suite():
    import doctest
    suite = doctest.DocFileSuite('tests/drawing/nx_pydot.txt',package='networkx')
    return suite


if __name__ == "__main__":
    import os
    import sys
    import unittest
    if sys.version_info[:2] < (2, 4):
        print "Python version 2.4 or later required for tests (%d.%d detected)." %  sys.version_info[:2]
        sys.exit(-1)
    # directory of networkx package (relative to this)
    nxbase=sys.path[0]+os.sep+os.pardir
    sys.path.insert(0,nxbase) # prepend to search path
    unittest.TextTestRunner().run(_test_suite())
    



#!/usr/bin/env python
"""
Read and write graphs.
"""
__author__ = """Aric Hagberg (hagberg@lanl.gov)"""
__date__ = "$Date: 2004-11-03 08:11:09 -0700 (Wed, 03 Nov 2004) $"
__credits__ = """"""
__revision__ = "$Revision: 503 $"
#    Copyright (C) 2004 by 
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Pieter Swart <swart@lanl.gov>
#    Distributed under the terms of the GNU Lesser General Public License
#    http://www.gnu.org/copyleft/lesser.html

from networkx import *

G = lollipop_graph(4,6)

pathlengths=[]

print "source vertex {target:length, }"
for v in G.nodes():
    spl=single_source_shortest_path_length(G,v)
    print v,spl
    for p in spl.values():
        pathlengths.append(p)

print
print "average shortest path length ", sum(pathlengths)/len(pathlengths)

# histogram of path lengths 
dist={}
for p in pathlengths:
    if dist.has_key(p):
        dist[p]+=1
    else:
        dist[p]=1

print
print "length #paths"
verts=dist.keys()
verts.sort()
for d in verts:
    print d,dist[d]

print "radius: ",radius(G)
print "diameter: ",diameter(G)
print "eccentricity: ",eccentricity(G,with_labels=True)
print "center: ",center(G)
print "periphery: ",periphery(G)
print "density: ", density(G)




"""
Cliques   - Find and manipulate cliques of graphs

Note that finding the largest clique of a graph has been
shown to be an NP complete problem so the algorithms here
could take a LONG time to run.  In practice it hasn't been 
too bad for the graphs tested.
"""
__author__ = """Dan Schult (dschult@colgate.edu)"""
__date__ = "$Date: 2005-06-15 07:56:03 -0600 (Wed, 15 Jun 2005) $"
__credits__ = """"""
__revision__ = "$Revision: 1021 $"
#    Copyright (C) 2004,2005 by 
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Pieter Swart <swart@lanl.gov>
#    Distributed under the terms of the GNU Lesser General Public License
#    http://www.gnu.org/copyleft/lesser.html

import networkx

def find_cliques(G):
   """
   Find_cliques algorithm based on Bron & Kerbosch
   
   This algorithm searchs for maximal cliques in a graph.
   maximal cliques are the largest complete subgraph containing
   a given point.  The largest maximal clique is sometimes called
   the maximum clique.
   
   This algorithm produces the list of maximal cliques each
   of which are a list of the members of the clique.
   
   Based on Algol algorithm published by Bron & Kerbosch
   A C version is available as part of the rambin package.
   http://www.ram.org/computing/rambin/rambin.html

   Reference::

      @article{362367,
       author = {Coen Bron and Joep Kerbosch},
       title = {Algorithm 457: finding all cliques of an undirected graph},
       journal = {Commun. ACM},
       volume = {16},
       number = {9},
       year = {1973},
       issn = {0001-0782},
       pages = {575--577},
       doi = {http://doi.acm.org/10.1145/362342.362367},
       publisher = {ACM Press},
       }
   """
   all_nodes=G.nodes()
   num_nodes=G.order()
   clique=[]
   cliques=[]
   _extend(all_nodes,0,num_nodes,G,clique,cliques)
   return cliques

def _extend(old_nodes,num_not,num_left,G,clique,cliques):
   """ 
   A recursive helper routine for find_cliques

   This recursive routine explores the "old_nodes" set of nodes
   for complete subgraphs.  num_left is the size of old_nodes. 
   The first num_not nodes are called the "not" group by B&K
   because they have already been considered. 
   The rest (num_left-num_not) of them are called "candidates".

   G is the graph, 
   clique is the clique being built,
   cliques is a list of maximal cliques found so far.
   """
   # set up sets nots and cands for easy reference
   cands=old_nodes[num_not:]
   nots=old_nodes[:num_not]

   #  Look for nodes that are "candidates" with the
   #  most neighbors among the "candidates"
   max_conn=0
   for p in cands:
      isneighbor=lambda v: G.has_edge(p,v)# "isneighbor" function
      conn=0       # count connections for ith node
      for nb in cands:   
        if isneighbor(nb):
           conn +=1
      if conn>max_conn:
         fixp=p
         max_conn=conn
         if max_conn > num_left-1: break  # found a most connected node
   
   # Note: if max_conn still zero, then no connections among candidates
   #     We can take a shortcut.
   #     If connects to a "not" node, we already got that clique
   #     Otherwise adding it makes a clique!
   if max_conn==0:
     for v in cands:
        not_conn=0
        isneighbor=lambda u:G.has_edge(v,u)  #"isneighbor" function
        for u in nots:
           if isneighbor(u):
              not_conn=1
              break
        if not_conn==0:
          clique.append(v)
##          print "New Maximal Clique Found!",clique
          cliques.append(clique[:])
          clique.remove(v)
     return

   # Set up the Best Candidate as our fixed point for this tree
   # best_position=old_nodes.index(fixp)
   fixpneighbor=lambda v: G.has_edge(fixp,v) #"isneighbor" function
   v=fixp

   # Go through each of the candidate 
   # nodes trying to build a clique
   #       cntr counts how many are left to check
   for cntr in range(num_left-num_not-max_conn,0,-1):
      # add the best node to the list of clique
##      print tt,"next candidate is:",v
      clique.append(v)
      isneighbor=lambda u: G.has_edge(v,u) # "isneighbor" function

      # Now create new lists to send to next recursion
      # fill new array--starting with "not" nodes
      new=[]
      for u in nots:
         if isneighbor(u):
            new.append(u)
      new_num_not=len(new)
      # now fill rest of new array with "candidate" nodes
      for u in cands:
         if isneighbor(u):
            new.append(u)
      new_num_left=len(new)

      # check if zero or one node left
      if new_num_left == 0:
         # Now there are no nodes left in "cand" or "not" to look at
         # we have found a maximal clique!!!
         cliques.append(clique[:])
##         print tt,"No nodes left so it must be a clique!"
##         print tt,"New Maximal Clique Found!",clique
      elif new_num_not<new_num_left:
         if new_num_left==1:
           #  Only one node left so it must be a clique!
           clique.append(new[0])
##           print tt,"Only one node left so it must be a clique!"
##           print tt,"New Maximal Clique Found!",clique
           cliques.append(clique[:])
           clique.remove(new[0])
         else:
           # Recursion on this routine
##            print tt,"Going into extend with cntr=",cntr
##            tt=tt + "  "
           _extend(new,new_num_not,new_num_left,G,clique,cliques)
##            tt=tt[0:len(tt)-2]  # go back to previous indentation 
##    else:    # new_num_not==new_num_left
##             # no new cliques so we're done with this candidate
##         print tt,"Only nodes from -not- left"


      # Done with checking branches for this node,
      # remove it from clique and add it to the NOT group
      clique.remove(v)
      nots.append(v)
      cands.remove(v)
      # find next candidate
      pos=0
      if cntr>1:
         while fixpneighbor(cands[pos]):
            pos += 1
         v=cands[pos]

def make_max_clique_graph(G,create_using=None,name=None):
    """ 
    Create the maximal clique graph of a graph.
    It finds the maximal cliques and treats these as nodes.
    The nodes are connected if they have common members in 
    the original graph.  Theory has done a lot with clique
    graphs, but I haven't seen much on maximal clique graphs.

    Note: This should be the same as make_clique_bipartite followed
    by project_up, but it saves all the intermediate stuff.
    """
    cliq=find_cliques(G)
    size=len(cliq)
    if create_using:
        B=create_using
        B.clear()
    else:
        B=networkx.Graph()
    if name is not None:
        B.name=name

    for i in range(1,size+1):
        B.add_node(i)
        cl=cliq[i-1]
        for j in range(i+1,size+1):
            other_cl=cliq[j-1]
            intersect=[ w for w in cl if w in other_cl ]
            if intersect:     # Not empty list
                B.add_edge(i,j)
    return B

def make_clique_bipartite(G,fpos=None,create_using=None,name=None):
    """ 
    Create a bipartite clique graph from a graph G.
    Nodes of G are retained as the "bottom nodes" of B and 
    cliques of G become "top nodes" of B.
    Edges are present if a bottom node belongs to the clique 
    represented by the top node.
 
    Returns a Graph with additional attribute B.node_type
    which is "Bottom" or "Top" appropriately.

    if fpos is not None, a second additional attribute B.pos
    is created to hold the position tuple of each node for viewing
    the bipartite graph.
    """
    cliq=find_cliques(G)
    if create_using:
        B=create_using
        B.clear()
    else:
        B=networkx.Graph()
    if name is not None:
        B.name=name

    B.add_nodes_from(G)
    B.node_type={}   # New Attribute for B
    for n in B:
        B.node_type[n]="Bottom"
 
    i=0
    if fpos is not None: 
       B.pos={}     # New Attribute for B
       delta_cpos=1./len(cliq)
       delta_ppos=1./G.order()
       cpos=0.
       ppos=0.
    for cl in cliq:
       i += 1
       name= -i   # Top nodes get negative names
       B.add_node(name)
       B.node_type[name]="Top"
       if fpos is not None: 
          if not B.pos.has_key(name):
             B.pos[name]=(0.2,cpos)
             cpos +=delta_cpos
       for v in cl:
          B.add_edge(name,v)
          if fpos is not None:
             if not B.pos.has_key(v):
                B.pos[v]=(0.8,ppos)
                ppos +=delta_ppos
    return B
   
def project_down(B,create_using=None,name=None):
    """ 
    Project a bipartite graph B down onto its "Bottom Nodes".
    The nodes retain their names and are connected if they
    share a common Top Node in the Bipartite Graph.
    Returns a Graph.
    """
    if create_using:
        G=create_using
        G.clear()
    else:
        G=networkx.Graph()
    if name is not None:
        G.name=name

    for v in B.nodes():
       if B.node_type[v]=="Bottom":
          G.add_node(v)
          for cv in B.neighbors(v):
             G.add_edges_from([(v,u) for u in B.neighbors(cv)])
    return G
 
def project_up(B,create_using=None,name=None):
    """ 
    Project a bipartite graph B up onto its "Top Nodes".
    The nodes retain their names and are connected if they
    share a common Bottom Node in the Bipartite Graph.
    Returns a Graph.
    """
    if create_using:
        G=create_using
        G.clear()
    else:
        G=networkx.Graph()
    if name is not None:
        G.name=name

    for v in B.nodes():
       if B.node_type[v]=="Top":
          vname= -v   #Change sign of name for Top Nodes
          G.add_node(vname)
          for cv in B.neighbors(v):
             # Note: -u changes the name (not Top node anymore)
             G.add_edges_from([(vname,-u) for u in B.neighbors(cv) if u!=v])
    return G
 
def graph_clique_number(G,cliques=None):
    """Return the clique number (size the largest clique) for G.
       Optional list of cliques can be input if already computed.
    """
    if cliques is None:
        cliques=find_cliques(G)
    return   max( [len(c) for c in cliques] )


def graph_number_of_cliques(G,cliques=None):
    """  Returns the number of maximal cliques in G
       Optional list of cliques can be input if already computed.
    """
    if cliques is None:
        cliques=find_cliques(G)
    return   len(cliques)

 
def node_clique_number(G,nodes=None,with_labels=False,cliques=None):
    """ Returns the size of the largest maximal clique containing
        each given node.  

        Returns a single or list depending on input nodes.
        Returns a dict keyed by node if "with_labels=True".
        Optional list of cliques can be input if already computed.
    """
    if nodes is None:                 # none, use entire graph
        nodes=G.nodes()
    elif  not isinstance(nodes, list):    # check for a list
        nodes=[nodes]             # assume it is a single value

    if cliques is None:
        cliques=find_cliques(G)
    d={}
    for v in nodes:
        d[v]=max([len(c) for c in cliques if v in c])
    
    if with_labels: return d
    if len(d)==1: return d[v] #return single value
    return d.values()


def number_of_cliques(G,nodes=None,cliques=None,with_labels=False):
    """ Returns the number of maximal cliques for each node.

        Returns a single or list depending on input nodes.
        Returns a dict keyed by node if "with_labels=True".
        Optional list of cliques can be input if already computed.
    """
    if nodes is None:
        nodes=G.nodes()   # none, get entire graph
    elif  not isinstance(nodes, list):    # check for a list
        nodes=[nodes]             # assume it is a single value

    if cliques is None:
        cliques=find_cliques(G)
    numcliq={}
    for v in nodes:
        numcliq[v]=len([1 for c in cliques if v in c])

    if with_labels: return numcliq
    if len(numcliq)==1: return numcliq[v]
    return numcliq.values()
    
def cliques_containing_node(G,nodes=None,cliques=None,with_labels=False):
    """ Returns a list of cliques containing the given node.

        Returns a single list or list of lists depending on input nodes.
        Returns a dict keyed by node if "with_labels=True".
        Optional list of cliques can be input if already computed.
    """
    if nodes is None:
        nodes=G.nodes()   # none, get entire graph
    elif  not isinstance(nodes, list):    # check for a list
        nodes=[nodes]             # assume it is a single value

    if cliques is None:
        cliques=find_cliques(G)
    vcliques={}
    for v in nodes:
        vcliques[v]=[c for c in cliques if v in c]

    if with_labels: return vcliques
    if len(vcliques)==1: return vcliques[v]
    return vcliques.values()

def _test_suite():
    import doctest
    suite = doctest.DocFileSuite('tests/cliques.txt',package='networkx')
    return suite


if __name__ == "__main__":
    import os   
    import sys
    import unittest
    if sys.version_info[:2] < (2, 4):
        print "Python version 2.4 or later required for tests (%d.%d detected)." %  sys.version_info[:2]
        sys.exit(-1)
    # directory of networkx package (relative to this)
    nxbase=sys.path[0]+os.sep+os.pardir
    sys.path.insert(0,nxbase) # prepend to search path
    unittest.TextTestRunner().run(_test_suite())
    



#!/usr/bin/env python
# uses docutils to execute python statements in documentation files
#
# python test.py tutorial.txt
#
# success means that all of the python comands in the documentation
# ran successfully and produced the correct output

import glob
import sys
import doctest
import unittest
import time
import os

if sys.version_info[:2] < (2, 4):
    print "Python version 2.4 or later required for tests (%d.%d detected)." %  sys.version_info[:2]
    sys.exit(-1)

nxbase=sys.path[0]+"/../.."  # directory of NX package (relative to this)
sys.path.insert(0,nxbase)    # prepend to search path

try:
    import networkx
except:
    print "Can't import networkx module"
    print sys.path
    raise
    

# argv is list of modules to test
sys.argv.pop(0)

files=[]
for a in sys.argv:
    files.extend(glob.glob(a))


print ('Testing networkx %s with Python %s on %s at %s'
       % (networkx.__version__, sys.version.split()[0],
          time.strftime('%Y-%m-%d'), time.strftime('%H:%M:%S')))
sys.stdout.flush()

for fname in files:
    suite = unittest.TestSuite()
    s = doctest.DocFileSuite(fname,module_relative=False)
    suite.addTest(s)
    runner = unittest.TextTestRunner()
    runner.run(suite)



"""
Read and write NetworkX graphs.

Note that NetworkX graphs can contain any hashable Python object as
node (not just integers and strings).  So writing a NetworkX graph
as a text file may not always be what you want: see write_gpickle
and gread_gpickle for that case.

This module provides the following :

Adjacency list with single line per node:
Useful for connected or unconnected graphs without edge data.

    write_adjlist(G, path)
    G=read_adjlist(path)

Adjacency list with multiple lines per node:
Useful for connected or unconnected graphs with or without edge data.

    write_multiline_adjlist(G, path)
    read_multiline_adjlist(path)

"""
__author__ = """Aric Hagberg (hagberg@lanl.gov)\nDan Schult (dschult@colgate.edu)"""
__date__ = """"""
__credits__ = """"""
__revision__ = ""
#    Copyright (C) 2004-2007 by 
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Pieter Swart <swart@lanl.gov>
#    Distributed under the terms of the GNU Lesser General Public License
#    http://www.gnu.org/copyleft/lesser.html

import cPickle 
import codecs
import locale
import string
import sys
import time

from networkx.utils import is_string_like,_get_fh
import networkx

def write_multiline_adjlist(G, path, delimiter=' ', comments='#'):
    """
    Write the graph G in multiline adjacency list format to the file
    or file handle path.

    See read_multiline_adjlist for file format details.

    >>> write_multiline_adjlist(G,"file.adjlist")

    path can be a filehandle or a string with the name of the file.

    >>> fh=open("file.adjlist")
    >>> write_multiline_adjlist(G,fh)

    Filenames ending in .gz or .bz2 will be compressed.

    >>> write_multiline_adjlist(G,"file.adjlist.gz")

    The file will use the default text encoding on your system.
    It is possible to write files in other encodings by opening
    the file with the codecs module.  See doc/examples/unicode.py
    for hints.

    >>> import codecs
    >>> fh=codecs.open("file.adjlist",encoding='utf=8') # use utf-8 encoding
    >>> write_multiline_adjlist(G,fh)

    """
    fh=_get_fh(path,mode='w')        
    pargs=comments+" "+string.join(sys.argv,' ')
    fh.write("%s\n" % (pargs))
    fh.write(comments+" GMT %s\n" % (time.asctime(time.gmtime())))
    fh.write(comments+" %s\n" % (G.name))

    # directed
    directed=G.is_directed()

    seen={}  # helper dict used to avoid duplicate edges
    for s in G.nodes():
        edges=[ edge for edge in G.edges_iter(s) if edge[1] not in seen ]
        deg=len(edges)
        if is_string_like(s):
            fh.write(s+delimiter)
        else:
            fh.write(str(s)+delimiter)                         
        fh.write("%i\n"%(deg))
        for edge in edges:
            t=edge[1]
            if len(edge)==2: # Graph or DiGraph
                d=None
            else:            # XGraph or XDiGraph
                d=edge[2]   # Note: could still be None
            if d is None:    
                if is_string_like(t):
                    fh.write(t+'\n')
                else:
                    fh.write(str(t)+'\n')                         
            else:   
                if is_string_like(t):
                    fh.write(t+delimiter)
                else:
                    fh.write(str(t)+delimiter)                         
                if is_string_like(d):
                    fh.write(d+"\n")
                else:
                    fh.write(str(d)+"\n")                         
        if not directed: 
            seen[s]=1
            
def read_multiline_adjlist(path, comments="#", delimiter=' ',
                           create_using=None,
                           nodetype=None, edgetype=None):
    """Read graph in multi-line adjacency list format from path.

    >>> G=read_multiline_adjlist("file.adjlist")

    path can be a filehandle or a string with the name of the file.

    >>> fh=open("file.adjlist")
    >>> G=read_multiline_adjlist(fh)

    Filenames ending in .gz or .bz2 will be compressed.

    >>> G=read_multiline_adjlist("file.adjlist.gz")

    nodetype is an optional function to convert node strings to nodetype

    For example

    >>> G=read_multiline_adjlist("file.adjlist", nodetype=int)

    will attempt to convert all nodes to integer type

    Since nodes must be hashable, the function nodetype must return hashable
    types (e.g. int, float, str, frozenset - or tuples of those, etc.) 

    edgetype is a function to convert edge data strings to edgetype

    >>> G=read_multiline_adjlist("file.adjlist", edgetype=int)

    create_using is an optional networkx graph type, the default is
    Graph(), a simple undirected graph 

    >>> G=read_multiline_adjlist("file.adjlist", create_using=DiGraph())

    The comments character (default='#') at the beginning of a
    line indicates a comment line.

    The entries are separated by delimiter (default=' ').
    If whitespace is significant in node or edge labels you should use
    some other delimiter such as a tab or other symbol.  
    

    Example multiline adjlist file format:: 

     # source target for Graph or DiGraph
     a 2
     b
     c
     d 1
     e

    or

     # source target for XGraph or XDiGraph with edge data
     a 2
     b edge-ab-data
     c edge-ac-data
     d 1
     e edge-de-data

    Reading the file will use the default text encoding on your system.
    It is possible to read files with other encodings by opening
    the file with the codecs module.  See doc/examples/unicode.py
    for hints.

    >>> import codecs
    >>> fh=codecs.open("file.adjlist", encoding='utf=8') # use utf-8 encoding
    >>> G=read_multiline_adjlist(fh)
    """
    if create_using is None:
        G=networkx.Graph()
    else:
        try:
            G=create_using
            G.clear()
        except:
            raise TypeError("Input graph is not a networkx graph type")

    # is this a XGraph or XDiGraph?
    if hasattr(G,'allow_multiedges')==True:
        xgraph=True
    else:
        xgraph=False

    inp=_get_fh(path)        

    for line in inp:
#        if line.startswith("#") or line.startswith("\n"):
#            continue
#        line=line.strip() #remove trailing \n 
        line = line[:line.find(comments)].strip()
        if not len(line): continue
        try:
            (u,deg)=line.split(delimiter)
            deg=int(deg)
        except:
            raise TypeError("Failed to read node and degree on line (%s)"%line)
        try:
            if nodetype is not None:
                u=nodetype(u)
        except:
            raise TypeError("Failed to convert node (%s) to type %s"\
                  %(u,nodetype))
        G.add_node(u)
        for i in range(deg):
            line=inp.next().strip()
            vlist=line.split(delimiter)
            if len(vlist)==1:
                v=vlist[0]
                d=None
            elif len(vlist)==2:
                (v,d)=vlist
            else:
                raise TypeError("Failed to read line: %s"%vlist)
            try:
                if nodetype is not None:
                    v=nodetype(v)
            except:
                raise TypeError("Failed to convert node (%s) to type %s"\
                                %(v,nodetype))
            if xgraph:
                if d is not None:
                    try:
                        if edgetype is not None:
                            d=edgetype(d)
                    except:
                        raise TypeError\
                              ("Failed to convert edge data (%s) to type %s"\
                                %(d, edgetype))
                G.add_edge(u,v,d)
            else:
                G.add_edge(u,v)
                
    return G


def write_adjlist(G, path, comments="#", delimiter=' '):
    """Write graph G in single-line adjacency-list format to path.

    See read_adjlist for file format details.

    >>> write_adjlist(G, "file.adjlist")

    path can be a filehandle or a string with the name of the file.

    >>> fh=open("file.adjlist")
    >>> write_adjlist(G, fh)

    Filenames ending in .gz or .bz2 will be compressed.

    >>> write_adjlist(G, "file.adjlist.gz")

    The file will use the default text encoding on your system.
    It is possible to write files in other encodings by opening
    the file with the codecs module.  See doc/examples/unicode.py
    for hints.

    >>> import codecs
    >>> fh=codecs.open("file.adjlist",encoding='utf=8') # use utf-8 encoding
    >>> write_adjlist(G,fh)

    Does not handle data in XGraph or XDiGraph, use 'write_edgelist'
    or 'write_multiline_adjlist'
    """
    fh=_get_fh(path,mode='w')        
    pargs=comments+" "+string.join(sys.argv,' ')
    fh.write("%s\n" % (pargs))
    fh.write(comments+" GMT %s\n" % (time.asctime(time.gmtime())))
    fh.write(comments+" %s\n" % (G.name))
    e={}  # helper dict used to avoid duplicate edges
    try:
        multiedges=G.multiedges
    except:
        multiedges=False

    # directed
    directed=G.is_directed()

    for s in G.nodes():
        if is_string_like(s):
            fh.write(s+delimiter)
        else:
            fh.write(str(s)+delimiter)
        for t in G.neighbors(s):
            if not directed:
                if e.has_key((t,s)):
                    continue
                e.setdefault((s,t),1)
            if multiedges:
                for d in G.get_edge(s,t):
                    if is_string_like(t):
                        fh.write(t+delimiter)
                    else:
                        fh.write(str(t)+delimiter)
            else:
                if is_string_like(t):
                    fh.write(t+delimiter)
                else:
                    fh.write(str(t)+delimiter)
        fh.write("\n")            


def read_adjlist(path, comments="#", delimiter=' ',
                 create_using=None, nodetype=None):
    """Read graph in single line adjacency list format from path.

    >>> G=read_adjlist("file.adjlist")

    path can be a filehandle or a string with the name of the file.

    >>> fh=open("file.adjlist")
    >>> G=read_adjlist(fh)

    Filenames ending in .gz or .bz2 will be compressed.

    >>> G=read_adjlist("file.adjlist.gz")

    nodetype is an optional function to convert node strings to nodetype

    For example

    >>> G=read_adjlist("file.adjlist", nodetype=int)

    will attempt to convert all nodes to integer type

    Since nodes must be hashable, the function nodetype must return hashable
    types (e.g. int, float, str, frozenset - or tuples of those, etc.) 

    create_using is an optional networkx graph type, the default is
    Graph(), a simple undirected graph 

    >>> G=read_adjlist("file.adjlist", create_using=DiGraph())

    Does not handle edge data: use 'read_edgelist' or 'read_multiline_adjlist'

    The comments character (default='#') at the beginning of a
    line indicates a comment line.

    The entries are separated by delimiter (default=' ').
    If whitespace is significant in node or edge labels you should use
    some other delimiter such as a tab or other symbol.  

     # source target
     a b c
     d e

    """
    if create_using is None:
        G=networkx.Graph()
    else:
        try:
            G=create_using
            G.clear()
        except:
            raise TypeError("Input graph is not a networkx graph type")

    fh=_get_fh(path)        

    for line in fh.readlines():
        line = line[:line.find(comments)].strip()
        if not len(line): continue
#        if line.startswith("#") or line.startswith("\n"):
#            continue
#        line=line.strip() #remove trailing \n 
        vlist=line.split(delimiter)
        u=vlist.pop(0)
        # convert types
        if nodetype is not None:
            try:
                u=nodetype(u)
            except:
                raise TypeError("Failed to convert node (%s) to type %s"\
                                %(u,nodetype))
        G.add_node(u)
        try:
            vlist=map(nodetype,vlist)
        except:
            raise TypeError("Failed to convert nodes (%s) to type %s"\
                            %(','.join(vlist),nodetype))
        for v in vlist:
            G.add_edge(u,v)
    return G



def _test_suite():
    import doctest
    suite = doctest.DocFileSuite('tests/readwrite/adjlist.txt',package='networkx')
    return suite


if __name__ == "__main__":
    import os 
    import sys
    import unittest
    if sys.version_info[:2] < (2, 4):
        print "Python version 2.4 or later required for tests (%d.%d detected)." %  sys.version_info[:2]
        sys.exit(-1)
    # directory of networkx package (relative to this)
    nxbase=sys.path[0]+os.sep+os.pardir
    sys.path.insert(0,nxbase) # prepend to search path
    unittest.TextTestRunner().run(_test_suite())
    



"""
Read graphs in GraphML format.
http://graphml.graphdrawing.org/

"""
# Original author: D. Eppstein, UC Irvine, August 12, 2003.
# The original code at http://www.ics.uci.edu/~eppstein/PADS/ is public domain.
__author__ = """Aric Hagberg (hagberg@lanl.gov)"""
__date__ = """"""
__credits__ = """"""
__revision__ = ""
#    Copyright (C) 2004-2007 by 
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Pieter Swart <swart@lanl.gov>
#    Distributed under the terms of the GNU Lesser General Public License
#    http://www.gnu.org/copyleft/lesser.html

import networkx
from networkx.exception import NetworkXException, NetworkXError
from networkx.utils import _get_fh
	
def read_graphml(path):
    """Read graph in GraphML format from path.
    Returns an XGraph or XDiGraph."""
    fh=_get_fh(path,mode='r')        
    G=parse_graphml(fh)
    return G
	
def parse_graphml(lines):
    """Read graph in GraphML format from string.
    Returns an XGraph or XDiGraph."""
    context = []
    G=networkx.XDiGraph()
    defaultDirected = [True]
	
    def start_element(name,attrs):
        context.append(name)
        if len(context) == 1:
            if name != 'graphml':
                raise GraphFormatError, \
                      'Unrecognized outer tag "%s" in GraphML' % name
        elif len(context) == 2 and name == 'graph':
            if 'edgedefault' not in attrs:
                raise GraphFormatError, \
                      'Required attribute edgedefault missing in GraphML'
            if attrs['edgedefault'] == 'undirected':
                print "undirected"
                defaultDirected[0] = False
        elif len(context) == 3 and context[1] == 'graph' and name == 'node':
            if 'id' not in attrs:
                raise GraphFormatError, 'Anonymous node in GraphML'
            G.add_node(attrs['id'])
        elif len(context) == 3 and context[1] == 'graph' and name == 'edge':
            if 'source' not in attrs:
                raise GraphFormatError, 'Edge without source in GraphML'
            if 'target' not in attrs:
                raise GraphFormatError, 'Edge without target in GraphML'
            G.add_edge(attrs['source'], attrs['target'],attrs.get('id'))  
            # no mixed graphs in NetworkX
            # handle mixed graphs by adding two directed
            # edges for an undirected edge in a directed graph 
            if attrs.get('directed')=='false':
                if defaultDirected[0]:
                    G.add_edge(attrs['target'], attrs['source'],attrs.get('id'))
		
    def end_element(name):
        context.pop()
	
    import xml.parsers.expat
    p = xml.parsers.expat.ParserCreate()
    p.StartElementHandler = start_element
    p.EndElementHandler = end_element
    for line in lines:
        p.Parse(line)
    p.Parse("", 1)

    if defaultDirected[0]:
        return G
    else:
        return G.to_undirected()


def _test_suite():
    import doctest
    suite = doctest.DocFileSuite('tests/readwrite/graphml.txt',
                                 package='networkx')
    return suite

if __name__ == "__main__":
    import os 
    import sys
    import unittest
    if sys.version_info[:2] < (2, 4):
        print "Python version 2.4 or later required for tests (%d.%d detected)." %  sys.version_info[:2]
        sys.exit(-1)
    # directory of networkx package (relative to this)
    nxbase=sys.path[0]+os.sep+os.pardir
    sys.path.insert(0,nxbase) # prepend to search path
    unittest.TextTestRunner().run(_test_suite())



"""
Functional interface to graph properties.
    
"""
__author__ = """Aric Hagberg (hagberg@lanl.gov)\nPieter Swart (swart@lanl.gov)\nDan Schult(dschult@colgate.edu)"""
#    Copyright (C) 2004-2006 by 
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Pieter Swart <swart@lanl.gov>
#    Distributed under the terms of the GNU Lesser General Public License
#    http://www.gnu.org/copyleft/lesser.html
#

# functional style helpers

def nodes(G):
    """Return a copy of the graph nodes in a list."""
    return G.nodes()

def nodes_iter(G):
    """Return an iterator over the graph nodes."""
    return G.nodes_iter()

def edges(G,nbunch=None):
    """Return list of  edges adjacent to nodes in nbunch.

    Return all edges if nbunch is unspecified or nbunch=None.

    For digraphs, edges=out_edges

    """
    return G.edges(nbunch)

def edges_iter(G,nbunch=None):
    """Return iterator over  edges adjacent to nodes in nbunch.

    Return all edges if nbunch is unspecified or nbunch=None.

    For digraphs, edges=out_edges
    
    """
    return G.edges_iter(nbunch)

def degree(G,nbunch=None,with_labels=False):
    """Return degree of single node or of nbunch of nodes.
    If nbunch is ommitted, then return degrees of *all* nodes.
    """
    return G.degree(nbunch,with_labels=with_labels)

def neighbors(G,n):
    """Return a list of nodes connected to node n. """
    return G.neighbors(n)

def number_of_nodes(G):
    """Return the order of a graph = number of nodes."""
    return G.number_of_nodes()

def number_of_edges(G):
    """Return the size of a graph = number of edges. """
    return G.number_of_edges()
    
def density(G):
    """Return the density of a graph.
    
    density = size/(order*(order-1)/2)
    density()=0.0 for an edge-less graph and 1.0 for a complete graph.
    """
    n=number_of_nodes(G)
    e=number_of_edges(G)
    if e==0: # includes cases n==0 and n==1
        return 0.0
    else:
        return e*2.0/float(n*(n-1))

def degree_histogram(G):
    """Return a list of the frequency of each degree value.
    
    The degree values are the index in the list.
    Note: the bins are width one, hence len(list) can be large
    (Order(number_of_edges))
    """
    degseq=G.degree()
    dmax=max(degseq)+1
    freq= [ 0 for d in xrange(dmax) ]
    for d in degseq:
        freq[d] += 1
    return freq

def is_directed(G):
    """ Return True if graph is directed."""
    return G.is_directed()


def _test_suite():
    import doctest
    suite = doctest.DocFileSuite('tests/function.txt',
                                 package='networkx')
    return suite


if __name__ == "__main__":
    import os
    import sys
    import unittest
    if sys.version_info[:2] < (2, 4):
        print "Python version 2.4 or later required for tests (%d.%d detected)." %  sys.version_info[:2]
        sys.exit(-1)
    # directory of networkx package (relative to this)
    nxbase=sys.path[0]+os.sep+os.pardir
    sys.path.insert(0,nxbase) # prepend to search path
    unittest.TextTestRunner().run(_test_suite())
    



#!/usr/bin/env python
"""
Routes to LANL from 186 sites on the Internet.

"""
__author__ = """Aric Hagberg (hagberg@lanl.gov)"""
__date__ = "$Date: 2005-04-01 11:51:23 -0700 (Fri, 01 Apr 2005) $"
__credits__ = """"""
__revision__ = ""
#    Copyright (C) 2004 by 
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Pieter Swart <swart@lanl.gov>
#    Distributed under the terms of the GNU Lesser General Public License
#    http://www.gnu.org/copyleft/lesser.html

import sys

def lanl_graph():
    """ Return the lanl internet view graph from lanl.edges
    """
    try:
        fh=open("lanl.edges","r")
    except IOError:
        print "lanl.edges not found"
        raise

    G=NX.Graph()

    time={}
    time[0]=0 # assign 0 to center node
    for line in fh.readlines():
        (head,tail,rtt)=line.split()
        G.add_edge(int(head),int(tail))
        time[int(head)]=float(rtt)

    # get largest component and assign ping times to G0time dictionary        
    G0=NX.connected_component_subgraphs(G)[0]   
    G0.rtt={}
    for n in G0:
        G0.rtt[n]=time[n]

    return G0

if __name__ == '__main__':

    import networkx as NX
    import math

    G=lanl_graph()


    print "graph has %d nodes with %d edges"\
          %(NX.number_of_nodes(G),NX.number_of_edges(G))
    print NX.number_connected_components(G),"connected components"

    try: # drawing
        import pylab as P

        P.figure(figsize=(8,8))
        # use graphviz to find radial layout
        pos=NX.graphviz_layout(G,prog="twopi",root=0)
        # draw nodes, coloring by rtt ping time
        NX.draw(G,pos,
                node_color=P.array([G.rtt[v] for v in G]),
                with_labels=False,
                alpha=0.5,
                node_size=15)
        P.savefig("lanl.png")
        print "Wrote lanl.png"
    except:
        print "This example could not find pylab or graphviz for drawing."
        pass



"""
An implementation of VF2 algorithm for graph ismorphism testing, as seen here:

   Luigi P. Cordella, Pasquale Foggia, Carlo Sansone, Mario Vento,
      "A (Sub)Graph Isomorphism Algorithm for Matching Large Graphs,"
      IEEE Transactions on Pattern Analysis and Machine Intelligence,
      vol. 26,  no. 10,  pp. 1367-1372,  Oct.,  2004.

Modified to handle undirected graphs.
Modified to handle multiple edges.
"""

__date__ = "$Date: 2007-08-21 16:49:09 -0600 (Tue, 21 Aug 2007) $"
__credits__ = "$Credits:$"
__revision__ = "$Revision: 680 $"

#    Copyright (C) 2007 by
#    Christopher Ellison <cellison@cse.ucdavis.edu>
#    Distributed under the terms of the GNU Lesser General Public License
#    http://www.gnu.org/copyleft/lesser.html
#
# This work was completed as part of the
# Computational Mechanics Python (CMPy) project.
# James P. Crutchfield, principal investigator.
# Center for Computational Science and Engineering and Physics Department,
# UC Davis.

import sys
from sets import Set

class GraphMatcher(object):
    """A GraphMatcher is responsible for matching undirected graphs (Graph or
    XGraph) in a  predetermined manner.  For graphs G1 and G2, this typically
    means a check for an isomorphism between them, though other checks are also
    possible.  For example, the GraphMatcher class can check if a subgraph 
    of G1 is isomorphic to G2.
    
    Matching is done via syntactic feasibility. It is also possible to check
    for semantic feasibility. Feasibility, then, is defined as the logical AND 
    of the two functions.  
    
    To include a semantic check, the GraphMatcher class should be subclassed,
    and the semantic_feasibility() function should be redefined.  By default, 
    the semantic feasibility function always returns True.  The effect of this
    is that semantics are not considered in the matching of G1 and G2. 
    
    For more information, see the docmentation for:
      syntactic_feasibliity()
      semantic_feasibility()
    
    Suppose G1 and G2 are isomorphic graphs. Verification is as follows:
    
    >>> GM = GraphMatcher(G1,G2)
    >>> GM.is_isomorphic()
    True
    >>> GM.mapping
    
    GM.mapping stores the isomorphism mapping.
    
    """
    def __init__(self, G1, G2):
        """Initialize GraphMatcher.
        
        Suppose G1 and G2 are undirected graphs.
        
        >>> GM = GraphMatcher(G1,G2)
        
        creates a GraphMatcher which only checks for syntactic feasibility.
        """
        self.G1 = G1
        self.G2 = G2
 
        # Set recursion limit.
        self.old_recursion_limit = sys.getrecursionlimit()
        expected_max_recursion_level = len(self.G2)
        if self.old_recursion_limit < 1.5 * expected_max_recursion_level:
            # Give some breathing room.
            sys.setrecursionlimit(int(1.5 * expected_max_recursion_level))
        
        # Declare that we will be searching for a graph-graph isomorphism.
        self.test = 'graph'

        # Initialize the isomorphism mapping.
        self.state = GMState(self)

    def __del__(self):
        # Restore the recursion limit
        sys.setrecursionlimit(self.old_recursion_limit)
                
    def candidate_pairs_iter(self):
        """This function returns an iterator over pairs to be considered
        for inclusion in the current partial isomorphism mapping."""

        # All computations are done using the current state!
        
        # First we compute the inout-terminal sets.
        T1_inout = [node for node in self.G1.nodes_iter() if (node in GMState.inout_1) and (node not in GMState.core_1)]
        T2_inout = [node for node in self.G2.nodes_iter() if (node in GMState.inout_2) and (node not in GMState.core_2)]
        
        # If T1_inout and T2_inout are both nonempty.
        # P(s) = T1_inout x {min T2_inout}
        if T1_inout and T2_inout:
            for node in T1_inout:
                yield node, min(T2_inout)
            
        else:
            # If T1_inout and T2_inout were both empty....
            # P(s) = (N_1 - M_1) x {min (N_2 - M_2)}
            if not (T1_inout or T2_inout):
                # First we determine the candidate node for G2
                other_node = min(Set(self.G2.nodes()) - Set(GMState.core_2))
                for node in self.G1:
                    if node not in GMState.core_1:
                        yield node, other_node

        # For all other cases, we don't have any candidate pairs.        

    def is_isomorphic(self):
        """Returns True if G1 and G2 are isomorphic graphs. Otherwise, it
        returns False."""

        # Declare that we are looking for a graph-graph isomorphism.
        self.test = 'graph'

        # Let's do two very quick checks!
        # QUESTION: Should we call faster_graph_could_be_isomorphic(G1,G2)?
        # For now, I just copy the code.
        
        # Check global properties
        if self.G1.order() != self.G2.order(): return False
    
        # Check local properties
        d1=self.G1.degree()
        d1.sort()
        d2=self.G2.degree()
        d2.sort()
        if d1 != d2: return False
        
        # Recall, self.match() will not return False.
        # It raises an exception or returns None
        try:
            self.match(self.state)
            return False
        except StopIteration:
            return True
        
    def match(self, state):
        """This function is called recursively to determine if a complete
        isomorphism can be found between G1 and G2.  It cleans up the class
        variables after each recursive call. If an isomorphism is found,
        we raise a StopIteration and jump immediately out of the recursion.
        """
        if len(GMState.core_1) == len(self.G2):
            # Save the final mapping, otherwise garbage collection deletes it.
            self.mapping = GMState.core_1.copy()
            # The mapping is complete.
            raise StopIteration
        else:
            for G1_node, G2_node in self.candidate_pairs_iter():
                if self.syntactic_feasibility(G1_node, G2_node):
                    if self.semantic_feasibility(G1_node, G2_node):
                        # Recursive call, adding the feasible state.
                        self.match(GMState(self, G1_node, G2_node))
            # Garbage collection for GMState() will 'restore data structures'.
    
    def semantic_feasibility(self, G1_node, G2_node):
        """The semantic feasibility function should return True if it is 
        acceptable to add the candidate pair (G1_node, G2_node) to the current 
        partial isomorphism mapping.   The logic should focus on semantic
        information contained in the edge data or a formalized node class.
        
        By acceptable, we mean that the subsequent mapping can still become a 
        complete isomorphism mapping.  Thus, if adding the candidate pair 
        definitely makes it so that the subsequent mapping cannot become a 
        complete isomorphism mapping, then this function must return False.
    
        The default semantic feasibility function always returns True. The 
        effect is that semantics are not considered in the matching of G1 
        and G2.

        The semantic checks might differ based on the what type of test is 
        being performed.  A keyword description of the test is stored in
        self.test.  Here is a quick description of the currently implemented
        tests:
        
          test='graph'    
            Indicates that the graph matcher is looking for a graph-graph
            isomorphism.
          test='subgraph'
            Indicates that the graph matcher is looking for a subgraph-graph
            isomorphism such that a subgraph of G1 is isomorphic to G2.
        
        Any subclass of GraphMatcher which redefines semantic_feasibility()
        must maintain the above form to keep the match() method functional.
        Implementation considerations should include directed and undirected
        graphs, as well as graphs with multiple edges.
        
        As an example, if edges have weights, one feasibility function would be
        to demand that the weight values/relationships are preserved in the 
        isomorphism mapping.        
        """
        return True
                    
    def subgraph_is_isomorphic(self):
        """Returns True if a subgraph of G1 is isomorphic to G2.  Otherwise,
        it returns False."""
        
        # Declare that we are looking for graph-subgraph isomorphism.
        self.test = 'subgraph'
        
        # Recall, self.match() will not return False.
        # It raises an exception or returns None
        try:
            self.match(self.state)
            return False
        except StopIteration:
            return True
        
    def syntactic_feasibility(self, G1_node, G2_node):
        """This function returns True if it is adding the candidate pair
        to the current partial isomorphism mapping is allowable.  The addition
        is allowable if the inclusion of the candidate pair does not make it
        impossible for an isomorphism to be found.
        """
        
        # The VF2 algorithm was designed to work with graphs having, at most,
        # only one edge connecting any two nodes.  This is not the case when
        # dealing with an XGraph or XDiGraph that has multiedges=True.
        # 
        # Basically, when we test the look-ahead rules R_pred and R_succ, we
        # will make sure that the number of edges are also taken into account.
        #
        # When multiedges=False, the value in the innermost dictionary is a 
        # singlet.  When multiedges=True, the value in the innermost dictionary
        # is a list.  However strange it may be, it is certainly possible to
        # have graphs which are isomorphic but with only one of them having
        # multiedges=True.  Thus, we must ALWAYS perform this check.
        
        
        
        ###
        ### Test at each step to get a return value as soon as possible.
        ###
        
        
        
        ### Look ahead 0
        
        # R_self

        # The number of selfloops for G1_node must equal the number of 
        # self-loops for G2_node. Without this check, we would fail on 
        # R_pred at the next recursion level. But it is good to prune the
        # search tree now.
        if self.G1.number_of_edges(G1_node,G1_node) != self.G2.number_of_edges(G2_node,G2_node):
            return False
                
        
        # R_neighbor
        
        # For each neighbor n' of n in the partial mapping, the corresponding
        # node m' is a neighbor of m, and vice versa. Also, the number of
        # edges must be equal.
        for neighbor in self.G1.adj[G1_node]:
            if neighbor in GMState.core_1:
                if not (GMState.core_1[neighbor] in self.G2.adj[G2_node]):
                    return False
                elif self.G1.number_of_edges(neighbor, G1_node) != self.G2.number_of_edges(GMState.core_1[neighbor], G2_node):
                    return False
        for neighbor in self.G2.adj[G2_node]:
            if neighbor in GMState.core_2:
                if not (GMState.core_2[neighbor] in self.G1.adj[G1_node]):
                    return False
                elif self.G1.number_of_edges(GMState.core_2[neighbor], G1_node) != self.G2.number_of_edges(neighbor, G2_node):
                    return False
        
        ### Look ahead 1
        
        # R_terminout
        # The number of neighbors of n that are in T_1^{inout} is equal to the
        # number of neighbors of m that are in T_2^{inout}, and vice versa.
        num1 = 0
        for neighbor in self.G1.adj[G1_node]:
            if (neighbor in GMState.inout_1) and (neighbor not in GMState.core_1):
                num1 += 1
        num2 = 0
        for neighbor in self.G2.adj[G2_node]:
            if (neighbor in GMState.inout_2) and (neighbor not in GMState.core_2):
                num2 += 1
        if self.test == 'graph':
            if not (num1 == num2):
                return False
        else: # self.test == 'subgraph'
            if not (num1 >= num2):
                return False


        ### Look ahead 2

        # R_new
        
        # The number of neighbors of n that are neither in the core_1 nor
        # T_1^{inout} is equal to the number of neighbors of m 
        # that are neither in core_2 nor T_2^{inout}.
        num1 = 0
        for neighbor in self.G1.adj[G1_node]:
            if neighbor not in GMState.inout_1:
                num1 += 1
        num2 = 0
        for neighbor in self.G2.adj[G2_node]:
            if neighbor not in GMState.inout_2:
                num2 += 1
        if self.test == 'graph':
            if not (num1 == num2):
                return False
        else: # self.test == 'subgraph'
            if not (num1 >= num2):
                return False
            
        # Otherwise, this node pair is syntactically feasible!
        return True
    
    
class DiGraphMatcher(object):
    """A DiGraphMatcher is responsible for matching directed graphs (DiGraph or
    XDiGraph) in a  predetermined manner.  For graphs G1 and G2, this typically
    means a check for an isomorphism between them, though other checks are also
    possible.  For example, the DiGraphMatcher class can check if a subgraph 
    of G1 is isomorphic to G2.
    
    Matching is done via syntactic feasibility. It is also possible to check
    for semantic feasibility. Feasibility, then, is defined as the logical AND 
    of the two functions.  
    
    To include a semantic check, you should subclass the GraphMatcher class and
    redefine semantic_feasibility().  By default, the semantic feasibility 
    function always returns True.  The effect of this is that semantics are not
    considered in the matching of G1 and G2. 
    
    For more information, see the docmentation for:
      syntactic_feasibliity()
      semantic_feasibility()

    
    
    Suppose G1 and G2 are isomorphic graphs. Verfication is as follows:
    
    >>> GM = GraphMatcher(G1,G2)
    >>> GM.is_isomorphic()
    True
    >>> GM.mapping
    
    GM.mapping stores the isomorphism mapping.
    
    """
        
    def __init__(self, G1, G2):
        """Initialize DiGraphMatcher.
        
        Suppose G1 and G2 are graphs.
        
        >>> GM = DiGraphMatcher(G1,G2)
        
        creates a DiGraphMatcher which only checks for syntactic feasibility.
        
        """
        self.G1 = G1
        self.G2 = G2
 
        # Set recursion limit.
        self.old_recursion_limit = sys.getrecursionlimit()
        expected_max_recursion_level = len(self.G2)
        if self.old_recursion_limit < 1.5 * expected_max_recursion_level:
            # Give some breathing room.
            sys.setrecursionlimit(int(1.5 * expected_max_recursion_level))
        
        # Declare that we will be searching for a graph-graph isomorphism.
        self.test = 'graph'
        
        # Initialize the isomorphism mapping.
        self.state = DiGMState(self)
        
        # Provide a convienient was to access the isomorphism mapping.
        self.mapping = DiGMState.core_1
               
    def __del__(self):
        # Restore the recursion limit
        sys.setrecursionlimit(self.old_recursion_limit)
        
    def candidate_pairs_iter(self):
        """This function returns an iterator over pairs to be considered
        for inclusion in the current partial isomorphism mapping."""
        
        # All computations are done using the current state!
        
        # First we compute the out-terminal sets.
        T1_out = [node for node in self.G1.nodes_iter() if (node in DiGMState.out_1) and (node not in DiGMState.core_1)]
        T2_out = [node for node in self.G2.nodes_iter() if (node in DiGMState.out_2) and (node not in DiGMState.core_2)]
        
        # If T1_out and T2_out are both nonempty.
        # P(s) = T1_out x {min T2_out}
        if T1_out and T2_out:
            node_2 = min(T2_out)
            for node_1 in T1_out:
                yield node_1, node_2
            
        else:
            # If T1_out and T2_out were both empty....
            # We compute the in-terminal sets.
            if not (T1_out or T2_out):
                T1_in = [node for node in self.G1.nodes_iter() if (node in DiGMState.in_1) and (node not in DiGMState.core_1)]
                T2_in = [node for node in self.G2.nodes_iter() if (node in DiGMState.in_2) and (node not in DiGMState.core_2)]
                
                # If T1_in and T2_in are both nonempty.
                # P(s) = T1_out x {min T2_out}
                if T1_in and T2_in:
                    node_2 = min(T2_in)
                    for node_1 in T1_in:
                        yield node_1, node_2
                else:
                    # If all terminal sets are empty...
                    # P(s) = (N_1 - M_1) x {min (N_2 - M_2)}
                    if not (T1_out or T2_out or T1_in or T2_in):
                        # First we determine the candidate node for G2
                        node_2 = min(Set(self.G2.nodes()) - Set(DiGMState.core_2))
                        for node_1 in self.G1:
                            if node_1 not in DiGMState.core_1:
                                yield node_1, node_2

        # For all other cases, we don't have any candidate pairs.        
        
    def is_isomorphic(self):
        """Returns True if G1 and G2 are isomorphic graphs. Otherwise, it
        returns False."""

        # Declare that we are looking for a graph-graph isomorphism.
        self.test = 'graph'

        # Let's do two very quick checks!
        # QUESTION: Should we call faster_graph_could_be_isomorphic(G1,G2)?
        # For now, I just copy the code.
        
        # Check global properties
        if self.G1.order() != self.G2.order(): return False
    
        # Check local properties
        d1=self.G1.degree()
        d1.sort()
        d2=self.G2.degree()
        d2.sort()
        if d1 != d2: return False
        
        # Recall, self.match() will not return False.
        # It raises an exception or returns None
        try:
            self.match(self.state)
            return False
        except StopIteration:
            return True
        
    def match(self, state):
        """This function is called recursively to determine if a complete
        isomorphism can be found between G1 and G2.  It cleans up the class
        variables after each recursive call. Because of this, this function
        will not return True or False. If a mapping is found, we will jump
        out of the recursion by throwing an exception.  Otherwise, we will
        return nothing.
        
        """
        if len(DiGMState.core_1) == len(self.G2):
            # Save the final mapping, otherwise garbage collection deletes it.
            self.mapping = DiGMState.core_1.copy()
            # The mapping is complete.
            raise StopIteration
        else:
            for G1_node, G2_node in self.candidate_pairs_iter():
                if self.syntactic_feasibility(G1_node, G2_node):
                    if self.semantic_feasibility(G1_node, G2_node):
                        # Recursive call, adding the feasible state.
                        self.match(DiGMState(self, G1_node,G2_node))
            # Garbage collection for DiGMState() will 'restore data structures'.


    def semantic_feasibility(self, G1_node, G2_node):
        """The semantic feasibility function should return True if it is 
        acceptable to add the candidate pair (G1_node, G2_node) to the current 
        partial isomorphism mapping.   The logic should focus on semantic
        information contained in the edge data or a formalized node class.
        
        By acceptable, we mean that the subsequent mapping can still become a 
        complete isomorphism mapping.  Thus, if adding the candidate pair 
        definitely makes it so that the subsequent mapping cannot become a 
        complete isomorphism mapping, then this function must return False.
    
        The default semantic feasibility function always returns True. The 
        effect is that semantics are not considered in the matching of
        G1 and G2.

        The semantic checks might differ based on the what type of test is 
        being performed.  A keyword description of the test is stored in
        self.test.  Here is a quick description of the currently implemented
        tests:
        
          test='graph'    
            Indicates that the graph matcher is looking for a graph-graph
            isomorphism.
          test='subgraph'
            Indicates that the graph matcher is looking for a subgraph-graph
            isomorphism such that a subgraph of G1 is isomorphic to G2.
        
        Any subclass of DiGraphMatcher which redefines semantic_feasibility()
        must maintain the above form to keep the match() method functional.
        Implementation considerations should include directed and undirected
        graphs, as well as graphs with multiple edges.
        
        As an example, if edges have weights, one feasibility function would be
        to demand that the weight values/relationships are preserved in the 
        isomorphism mapping.
        
        """
        return True
                    
    def subgraph_is_isomorphic(self):
        """Returns True if a subgraph of G1 is isomorphic to G2.  Otherwise,
        it returns False."""
        
        # Declare that we are looking for graph-subgraph isomorphism.
        self.test = 'subgraph'
        
        # Recall, self.match() will not return False.
        # It raises an exception or returns None
        try:
            self.match(self.state)
            return False
        except StopIteration:
            return True

    def syntactic_feasibility(self, G1_node, G2_node):
        """This function returns True if it is adding the candidate pair
        to the current partial isomorphism mapping is allowable.
        
        Keywords:
          test='graph'    
              Checks for graph-graph isomorphism. This is the default value.
          test='subgraph'
              Checks for graph-subgraph isomorphism in such a way that
              a subgraph of G1 might be isomorphic to G2.
        
        """
        
        # The VF2 algorithm was designed to work with graphs that have, at most
        # only one edge connecting any two nodes.  This is not the case when
        # dealing with an XGraph or XDiGraph that has multiedges=True.
        # 
        # Basically, when we test the look-ahead rules R_pred and R_succ, 
        # we will make sure that the number of edges are also considered.
        #
        # We also have R_self, a test that performs the edge count on the 
        # candidate nodes themselves.
        #
        # When multiedges=False, the value in the innermost dictionary is a 
        # singlet.  When multiedges=True, the value in the innermost dictionary
        # is a list.  However strange it may be, it is certainly possible to
        # have graphs which are isomorphic but with only one of them having
        # multiedges=True.  Thus, we must ALWAYS perform this check, and we
        # must perform the check separately for each graph.
        
        
        
        ###
        ### Test at each step to get a return value as soon as possible.
        ###
        
        
        
        ### Look ahead 0
        
        # R_self
        
        # The number of selfloops for G1_node must equal the number of
        # self-loops for G2_node. Without this check, we would fail on R_pred
        # at the next recursion level. This should prune the tree even further.
        
        if self.G1.number_of_edges(G1_node,G1_node) != self.G2.number_of_edges(G2_node,G2_node):
            return False

        ### predecessors_iter, successors_iter and neighbors_iter does not 
        ### behave how we need it to. With multiedges, it returns the same
        ### node multiple times.  The result is that we do the same check
        ### repeatedly.  If NetworkX changes this behavior, we can go back to
        ### predecessors_iter (etc), but for now, we must access the underlying
        ### structure. For example, 
        ###   self.G1.pred[G1_node] 
        ### vs 
        ###   self.G1.predecessors_iter(G1_node)


        # R_pred
        
        # For each predecessor n' of n in the partial mapping, the
        # corresponding node m' is a predecessor of m, and vice versa. Also,
        # the number of edges must be equal.
        for predecessor in self.G1.pred[G1_node]:
            if predecessor in DiGMState.core_1:
                if not (DiGMState.core_1[predecessor] in self.G2.pred[G2_node]):
                    return False
                elif self.G1.number_of_edges(predecessor, G1_node) != self.G2.number_of_edges(DiGMState.core_1[predecessor], G2_node):
                    return False
                
        for predecessor in self.G2.pred[G2_node]:
            if predecessor in DiGMState.core_2:
                if not (DiGMState.core_2[predecessor] in self.G1.pred[G1_node]):
                    return False
                elif self.G1.number_of_edges(DiGMState.core_2[predecessor], G1_node) != self.G2.number_of_edges(predecessor, G2_node):
                    return False


        # R_succ

        # For each successor n' of n in the partial mapping, the corresponding 
        # node m' is a successor of m, and vice versa. Also, the number of
        # edges must be equal.
        for successor in self.G1.succ[G1_node]:
            if successor in DiGMState.core_1:
                if not (DiGMState.core_1[successor] in self.G2.succ[G2_node]):
                    return False
                elif self.G1.number_of_edges(G1_node, successor) != self.G2.number_of_edges(G2_node, DiGMState.core_1[successor]):
                    return False
                                
        for successor in self.G2.succ[G2_node]:
            if successor in DiGMState.core_2:
                if not (DiGMState.core_2[successor] in self.G1.succ[G1_node]):
                    return False
                elif self.G1.number_of_edges(G1_node, DiGMState.core_2[successor]) != self.G2.number_of_edges(G2_node, successor):
                    return False

        
        ### Look ahead 1
        
        # R_termin
        # The number of predecessors of n that are in T_1^{in} is equal to the
        # number of predecessors of m that are in T_2^{in}.
        num1 = 0
        for predecessor in self.G1.pred[G1_node]:
            if (predecessor in DiGMState.in_1) and (predecessor not in DiGMState.core_1):
                num1 += 1
        num2 = 0
        for predecessor in self.G2.pred[G2_node]:
            if (predecessor in DiGMState.in_2) and (predecessor not in DiGMState.core_2):
                num2 += 1
        if self.test == 'graph':
            if not (num1 == num2):
                return False
        else: # self.test == 'subgraph'
            if not (num1 >= num2):
                return False

        # The number of successors of n that are in T_1^{in} is equal to the
        # number of successors of m that are in T_2^{in}.
        num1 = 0
        for successor in self.G1.succ[G1_node]:
            if (successor in DiGMState.in_1) and (successor not in DiGMState.core_1):
                num1 += 1
        num2 = 0
        for successor in self.G2.succ[G2_node]:
            if (successor in DiGMState.in_2) and (successor not in DiGMState.core_2):
                num2 += 1                
        if self.test == 'graph':
            if not (num1 == num2):
                return False
        else: # self.test == 'subgraph'
            if not (num1 >= num2):
                return False

        # R_termout
        
        # The number of predecessors of n that are in T_1^{out} is equal to the
        # number of predecessors of m that are in T_2^{out}.
        num1 = 0
        for predecessor in self.G1.pred[G1_node]:
            if (predecessor in DiGMState.out_1) and (predecessor not in DiGMState.core_1):
                num1 += 1
        num2 = 0
        for predecessor in self.G2.pred[G2_node]:
            if (predecessor in DiGMState.out_2) and (predecessor not in DiGMState.core_2):
                num2 += 1
        if self.test == 'graph':
            if not (num1 == num2):
                return False
        else: # self.test == 'subgraph'
            if not (num1 >= num2):
                return False

        # The number of successors of n that are in T_1^{out} is equal to the
        # number of successors of m that are in T_2^{out}.
        num1 = 0
        for successor in self.G1.succ[G1_node]:
            if (successor in DiGMState.out_1) and (successor not in DiGMState.core_1):
                num1 += 1
        num2 = 0
        for successor in self.G2.succ[G2_node]:
            if (successor in DiGMState.out_2) and (successor not in DiGMState.core_2):
                num2 += 1                                
        if self.test == 'graph':
            if not (num1 == num2):
                return False
        else: # self.test == 'subgraph'
            if not (num1 >= num2):
                return False
        
        ### Look ahead 2

        # R_new
        
        # The number of predecessors of n that are neither in the core_1 nor
        # T_1^{in} nor T_1^{out} is equal to the number of predecessors of m 
        # that are neither in core_2 nor T_2^{in} nor T_2^{out}.
        num1 = 0
        for predecessor in self.G1.pred[G1_node]:
            if (predecessor not in DiGMState.in_1) and (predecessor not in DiGMState.out_1):
                num1 += 1
        num2 = 0
        for predecessor in self.G2.pred[G2_node]:
            if (predecessor not in DiGMState.in_2) and (predecessor not in DiGMState.out_2):
                num2 += 1
        if self.test == 'graph':
            if not (num1 == num2):
                return False
        else: # self.test == 'subgraph'
            if not (num1 >= num2):
                return False
            
        # The number of successors of n that are neither in the core_1 nor
        # T_1^{in} nor T_1^{out} is equal to the number of successors of m 
        # that are neither in core_2 nor T_2^{in} nor T_2^{out}.
        num1 = 0
        for successor in self.G1.succ[G1_node]:
            if (successor not in DiGMState.in_1) and (successor not in DiGMState.out_1):
                num1 += 1
        num2 = 0
        for successor in self.G2.succ[G2_node]:
            if (successor not in DiGMState.in_2) and (successor not in DiGMState.out_2):
                num2 += 1
        if self.test == 'graph':
            if not (num1 == num2):
                return False
        else: # self.test == 'subgraph'
            if not (num1 >= num2):
                return False
        
        # Otherwise, this node pair is syntactically feasible!
        return True
    
        
class GMState(object):
    """This class is used internally by the GraphMatcher class.  It is used
    only to store state specific data. There will be at most G2.order() of
    these objects in memory at a time, due to the depth-first search
    strategy employed by the VF2 algorithm.
    """
    
    ###
    ### The following variables are class variables.
    ### So they will be shared by all instances of the GMState class.
    ### This is the improvement of the VF2 algorithm over the VF algorithm.
    ###
    
    # core_1[n] contains the index of the node paired with n, which is m,
    #           provided n is in the mapping.
    # core_2[m] contains the index of the node paired with m, which is n,
    #           provided m is in the mapping.
    core_1 = {}
    core_2 = {}
    
    # See the paper for definitions of M_x and T_x^{y}
    
    # inout_1[n]  is non-zero if n is in M_1 or in T_1^{inout}
    # inout_2[m]  is non-zero if m is in M_2 or in T_2^{inout}
    #
    # The value stored is the depth of the SSR tree when the node became
    # part of the corresponding set.
    inout_1 = {}
    inout_2 = {}
    # Practically, these sets simply store the nodes in the subgraph.
    
    def __init__(self, GM, G1_node=None, G2_node=None):
        """Initializes GMState object.
        
        Pass in the GraphMatcher to which this DiGMState belongs and the
        new node pair that will be added to the GraphMatcher's current
        isomorphism mapping.
        """
        # Initialize the last stored node pair.
        self.G1_node = None
        self.G2_node = None
        self.depth = len(GMState.core_1)
      
        # Watch out! G1_node == 0 should evaluate to True.
        if G1_node is not None and G2_node is not None:
            # Add the node pair to the isomorphism mapping.
            GMState.core_1[G1_node] = G2_node
            GMState.core_2[G2_node] = G1_node

            # Store the node that was added last.
            self.G1_node = G1_node
            self.G2_node = G2_node
            
            # Now we must update the other two vectors.
            # We will add only if it is not in there already!
            self.depth = len(GMState.core_1)
            
            # First we add the new nodes...
            if G1_node not in GMState.inout_1:
                GMState.inout_1[G1_node] = self.depth
            if G2_node not in GMState.inout_2:
                    GMState.inout_2[G2_node] = self.depth
                    
            # Now we add every other node...
            
            # Updates for T_1^{inout}
            new_nodes = Set([])
            for node in GMState.core_1:
                new_nodes.update([neighbor for neighbor in GM.G1.neighbors(node) if neighbor not in GMState.core_1])
            for node in new_nodes:
                if node not in GMState.inout_1:
                    GMState.inout_1[node] = self.depth

            # Updates for T_2^{inout}
            new_nodes = Set([])
            for node in GMState.core_2:
                new_nodes.update([neighbor for neighbor in GM.G2.neighbors(node) if neighbor not in GMState.core_2])
            for node in new_nodes:
                if node not in GMState.inout_2:
                    GMState.inout_2[node] = self.depth
                
    def __del__(self):
        """Deletes the GMState object and restores the class variables."""
        
        # First we remove the node that was added from the core vectors.
        # Watch out! G1_node == 0 should evaluate to True.
        if self.G1_node is not None and self.G2_node is not None:
            del GMState.core_1[self.G1_node]
            del GMState.core_2[self.G2_node]

        # Now we revert the other two vectors.        
        # Thus, we delete all entries which have this depth level.
        for vector in (GMState.inout_1, GMState.inout_2):
            for node in vector.keys():
                if vector[node] == self.depth:
                    del vector[node]
                    
    

class DiGMState(object):
    """This class is used internally by the DiGraphMatcher class.  It is used
    only to store state specific data. There will be at most G2.order() of
    these objects in memory at a time, due to the depth-first search
    strategy employed by the VF2 algorithm.
    """
    
    ###
    ### The following variables are class variables.
    ### So they will be shared by all instances of the DiGMState class.
    ### This is the improvement of the VF2 algorithm over the VF algorithm.
    ###
    
    # core_1[n] contains the index of the node paired with n, which is m,
    #           provided n is in the mapping.
    # core_2[m] contains the index of the node paired with m, which is n,
    #           provided m is in the mapping.
    core_1 = {}
    core_2 = {}
    
    # See the paper for definitions of M_x and T_x^{y}
    
    # in_1[n]  is non-zero if n is in M_1 or in T_1^{in}
    # out_1[n] is non-zero if n is in M_1 or in T_1^{out}
    #
    # in_2[m]  is non-zero if m is in M_2 or in T_2^{in}
    # out_2[m] is non-zero if m is in M_2 or in T_2^{out}
    #
    # The value stored is the depth of the search tree when the node became
    # part of the corresponding set.
    in_1 = {}
    in_2 = {}
    out_1 = {}
    out_2 = {}
    
    def __init__(self, DiGM, G1_node=None, G2_node=None):
        """Initializes DiGMState object.
        
        Pass in the DiGraphMatcher to which this DiGMState belongs and the
        new node pair that will be added to the GraphMatcher's current
        isomorphism mapping.
        """
        # Initialize the last stored node pair.
        self.G1_node = None
        self.G2_node = None
        self.depth = len(DiGMState.core_1)
      
        # Watch out! G1_node == 0 should evaluate to True.
        if G1_node is not None and G2_node is not None:
            # Add the node pair to the isomorphism mapping.
            DiGMState.core_1[G1_node] = G2_node
            DiGMState.core_2[G2_node] = G1_node
            
            # Store the node that was added last.
            self.G1_node = G1_node
            self.G2_node = G2_node
            
            # Now we must update the other four vectors.
            # We will add only if it is not in there already!
            self.depth = len(DiGMState.core_1)
            
            # First we add the new nodes...
            for vector in (DiGMState.in_1, DiGMState.out_1):
                if G1_node not in vector:
                    vector[G1_node] = self.depth
            for vector in (DiGMState.in_2, DiGMState.out_2):
                if G2_node not in vector:
                    vector[G2_node] = self.depth
                    
            # Now we add every other node...
            
            # Updates for T_1^{in}
            new_nodes = Set([])
            for node in DiGMState.core_1:
                new_nodes.update([predecessor for predecessor in DiGM.G1.predecessors(node) if predecessor not in DiGMState.core_1])
            for node in new_nodes:
                if node not in DiGMState.in_1:
                    DiGMState.in_1[node] = self.depth
                
            # Updates for T_2^{in}
            new_nodes = Set([])
            for node in DiGMState.core_2:
                new_nodes.update([predecessor for predecessor in DiGM.G2.predecessors(node) if predecessor not in DiGMState.core_2])
            for node in new_nodes:
                if node not in DiGMState.in_2:
                    DiGMState.in_2[node] = self.depth
                
            # Updates for T_1^{out}
            new_nodes = Set([])        
            for node in DiGMState.core_1:
                new_nodes.update([successor for successor in DiGM.G1.successors(node) if successor not in DiGMState.core_1])
            for node in new_nodes:
                if node not in DiGMState.out_1:                
                    DiGMState.out_1[node] = self.depth
    
            # Updates for T_2^{out}
            new_nodes = Set([])        
            for node in DiGMState.core_2:
                new_nodes.update([successor for successor in DiGM.G2.successors(node) if successor not in DiGMState.core_2])
            for node in new_nodes:
                if node not in DiGMState.out_2:
                    DiGMState.out_2[node] = self.depth

    def __del__(self):
        """Deletes the DiGMState object and restores the class variables."""
        
        # First we remove the node that was added from the core vectors.
        # Watch out! G1_node == 0 should evaluate to True.
        if self.G1_node is not None and self.G2_node is not None:
            del DiGMState.core_1[self.G1_node]
            del DiGMState.core_2[self.G2_node]

        # Now we revert the other four vectors.        
        # Thus, we delete all entries which have this depth level.
        for vector in (DiGMState.in_1, DiGMState.in_2, DiGMState.out_1, DiGMState.out_2):
            for node in vector.keys():
                if vector[node] == self.depth:
                    del vector[node]
                    
                    
def _test_suite():
    import doctest
    suite = doctest.DocFileSuite('tests/isomorphvf2.txt',package='networkx')
    return suite

if __name__ == "__main__":
    import os
    import sys
    import unittest
    if sys.version_info[:2] < (2, 4):
        print "Python version 2.4 or later required for tests (%d.%d detected)." %  sys.version_info[:2]
        sys.exit(-1)
    # directory of networkx package (relative to this)
    nxbase=sys.path[0]+os.sep+os.pardir
    sys.path.insert(0,nxbase) # prepend to search path
    unittest.TextTestRunner().run(_test_suite())
    



#!/usr/bin/env python
"""
Random graph from given degree sequence.
"""
__author__ = """Aric Hagberg (hagberg@lanl.gov)"""
__date__ = "$Date: 2004-11-03 08:11:09 -0700 (Wed, 03 Nov 2004) $"
__credits__ = """"""
__revision__ = "$Revision: 503 $"
#    Copyright (C) 2004 by 
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Pieter Swart <swart@lanl.gov>
#    Distributed under the terms of the GNU Lesser General Public License
#    http://www.gnu.org/copyleft/lesser.html

from networkx import *
from networkx.generators.degree_seq import *

z=[5,3,3,3,3,2,2,2,1,1,1]
is_valid_degree_sequence(z)

print "Configuration model"
G=configuration_model(z)  # configuration model
degree_sequence=degree(G) # degree sequence
print "Degree sequence", degree_sequence
print "Degree histogram"
hist={}
for d in degree_sequence:
    if hist.has_key(d):
        hist[d]+=1
    else:
        hist[d]=1
print "degree #nodes"
for d in hist:
    print d,hist[d]





#!/usr/bin/env python

import glob
import os
import sys
import doctest
import unittest

def all():
    try:
        from pkg_resources import resource_filename, resource_listdir
        tests=[resource_filename(__name__, t)
              for t in resource_listdir("networkx",'tests') if t.endswith("txt")]
        tests+=[resource_filename(__name__, 'generators/'+t)
              for t in resource_listdir("networkx",'tests/generators') if t.endswith("txt")]
        tests+=[resource_filename(__name__, 'readwrite/'+t)
              for t in resource_listdir("networkx",'tests/readwrite') if t.endswith("txt")]
    except:
        import networkx
        base=os.path.dirname(networkx.__file__)+"/tests/"
        tests=glob.glob(base+"*.txt") 
        tests+=glob.glob(base+"generators/*.txt")
        tests+=glob.glob(base+"readwrite/*.txt")
        #tests+=glob.glob("drawing/*.txt")  

    # tests depending on numpy
    try:
        import numpy
    except ImportError:
        print "numpy not found: skipping tests of spectrum.py, threshold.py, convert.py (numpy)"
        tests=[t for t in tests \
               if 'spectrum.txt' not in t \
               if 'threshold.txt' not in t\
               if 'convert_numpy.txt' not in t\
               ]

    # tests depending on scipy        
    try:
        import scipy
    except ImportError:
        print "scipy not found: skipping tests of convert.py (scipy)"
        tests=[t for t in tests \
               if 'convert_scipy.txt' not in t\
               ]
    # tests depending on yaml        
    try:
        import yaml
    except ImportError:
        print "yaml not found: skipping tests of nx_yaml.py (yaml)"
        tests=[t for t in tests \
               if 'nx_yaml.txt' not in t\
               ]
    # tests depending on pyparsing
    try:
        import pyparsing
    except ImportError:
        print "pyparsing not found: skipping tests of gml.py"
        tests=[t for t in tests \
               if 'gml.txt' not in t\
               ]





    suite = unittest.TestSuite()
    for t in tests:
        s = doctest.DocFileSuite(t,module_relative=False)
        suite.addTest(s)
    return suite

def run():
    if sys.version_info[:2] < (2, 4):
        print "Python version 2.4 or later required for tests (%d.%d detected)." %  sys.version_info[:2]
        sys.exit(-1)
    runner = unittest.TextTestRunner()
    runner.run(all())
    

if __name__ == "__main__":
    try:
        import networkx
    except:
        print "Can't import networkx module, not in path"
        print sys.path
        raise
    
    run()



#!/usr/bin/env python
"""
Draw a graph with matplotlib, color by degree.
You must have matplotlib for this to work.
"""
__author__ = """Aric Hagberg (hagberg@lanl.gov)"""
__date__ = "$Date: 2005-03-22 13:57:46 -0700 (Tue, 22 Mar 2005) $"
__credits__ = """"""
__revision__ = "$Revision: 831 $"
#    Copyright (C) 2004 by 
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Pieter Swart <swart@lanl.gov>
#    Distributed under the terms of the GNU Lesser General Public License
#    http://www.gnu.org/copyleft/lesser.html

try:
    from pylab import *
except:
    print "pylab not found: see https://networkx.lanl.gov/Drawing.html for info"
    raise 
    
from networkx import *


G=barbell_graph(4,3) # 4x4 grid
pos=spring_layout(G) # positions for all nodes
# nodes
greennodes=[0,1,2,3]
bluenodes=[7,8,9,10]
othernodes=[v for v in G if v not in greennodes+bluenodes]
draw_networkx_nodes(G,pos,nodelist=greennodes,node_color='g',
                    node_size=500)
draw_networkx_nodes(G,pos,nodelist=bluenodes,node_color='b',
                    alpha=0.5,node_size=100)
draw_networkx_nodes(G,pos,nodelist=othernodes)

# edges
draw_networkx_edges(G,pos,width=2.0)
edgelist=[(3,4),(4,5),(5,6),(6,7)]
draw_networkx_edges(G,pos,edgelist=edgelist, width=8,alpha=0.5,edge_color='g')

# labels
labels={}
labels[1]='one'
labels[10]='ten'
draw_networkx_labels(G,pos,labels,font_color='w',font_family='sans-serif')

# math labels
labels={}
labels[4]='$\pi$'
labels[5]='$\mu$'
labels[6]='$\epsilon$'
draw_networkx_labels(G,pos,labels,font_size=16)

savefig("draw_fancy.png") # save as png
show() # display



"""
Base exceptions and errors for NetworkX.
    
"""
__author__ = """Aric Hagberg (hagberg@lanl.gov)\nPieter Swart (swart@lanl.gov)\nDan Schult(dschult@colgate.edu)"""
#    Copyright (C) 2004-2006 by 
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Pieter Swart <swart@lanl.gov>
#    Distributed under the terms of the GNU Lesser General Public License
#    http://www.gnu.org/copyleft/lesser.html
#

# Exception handling

# the root of all Exceptions
class NetworkXException(Exception):
    """Base class for exceptions in NetworkX."""

class NetworkXError(NetworkXException):
    """Exception for a serious error in NetworkX"""


def _test_suite():
    import doctest
    suite = doctest.DocFileSuite('tests/exception.txt',
                                 package='networkx')
    return suite


if __name__ == "__main__":
    import os
    import sys
    import unittest
    if sys.version_info[:2] < (2, 4):
        print "Python version 2.4 or later required for tests (%d.%d detected)." %  sys.version_info[:2]
        sys.exit(-1)
    # directory of networkx package (relative to this)
    nxbase=sys.path[0]+os.sep+os.pardir
    sys.path.insert(0,nxbase) # prepend to search path
    unittest.TextTestRunner().run(_test_suite())
    



"""
A package for reading and writing graphs in various formats.

"""

#    Copyright (C) 2004-2007 by 
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Pieter Swart <swart@lanl.gov>
#    Distributed under the terms of the GNU Lesser General Public License
#    http://www.gnu.org/copyleft/lesser.html
#


from adjlist import  read_multiline_adjlist, write_multiline_adjlist, \
   read_adjlist, write_adjlist

from edgelist import read_edgelist,write_edgelist
from gpickle import read_gpickle,write_gpickle
from nx_yaml import read_yaml, write_yaml
from graphml import read_graphml, parse_graphml
from leda import read_leda, parse_leda
from sparsegraph6 import read_graph6,parse_graph6,read_graph6_list,\
     read_sparse6,parse_sparse6,read_sparse6_list
from gml import read_gml,parse_gml,write_gml



#!/usr/bin/env python

from glob import glob
import os
for filename in glob('*.txt'):
    basename=os.path.splitext(filename)[0]  # get extension
    print filename,basename+'.html'
    os.system("rst2html.py %s > %s.html"%(filename,basename))

os.system("epydoc -v --docformat=restructuredtext -o reference networkx")



"""
Generators for some directed graphs.

gn_graph: growing network 
gnc_graph: growing network with copying
gnr_graph: growing network with redirection

"""
#    Copyright (C) 2006 by 
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Pieter Swart <swart@lanl.gov>
#    Distributed under the terms of the GNU Lesser General Public License
#    http://www.gnu.org/copyleft/lesser.html
__author__ ="""Aric Hagberg (hagberg@lanl.gov)"""

import math
import random

from networkx.digraph import DiGraph
from networkx.generators.classic import empty_graph
from networkx.utils import discrete_sequence


def gn_graph(n,kernel=lambda x:x ,seed=None):
    """Return the GN (growing network) digraph with n nodes.

    The graph is built by adding nodes one at a time with a link
    to one previously added node.  The target node for the link is chosen
    with probability based on degree.  The default attachment kernel is
    a linear function of degree.

    The graph is always a (directed) tree.

    Example:

    >>> D=gn_graph(10)       # the GN graph
    >>> G=D.to_undirected()  # the undirected version

    To specify an attachment kernel use the kernel keyword

    >>> D=gn_graph(10,kernel=lambda x:x**1.5) # A_k=k^1.5

    Reference::

      @article{krapivsky-2001-organization,
      title   = {Organization of Growing Random Networks},
      author  = {P. L. Krapivsky and S. Redner},
      journal = {Phys. Rev. E},
      volume  = {63},
      pages   = {066123},
      year    = {2001},
      }


    """
    G=empty_graph(1,create_using=DiGraph())
    G.name="gn_graph(%s)"%(n)

    if seed is not None:
        random.seed(seed)

    if n==1:
        return G

    G.add_edge(1,0) # get started
    ds=[1,1] # degree sequence

    for source in range(2,n):
        # compute distribution from kernel and degree
        dist=[kernel(d) for d in ds] 
        # choose target from discrete distribution 
        target=discrete_sequence(1,distribution=dist)[0]
        G.add_edge(source,target)
        ds.append(1)  # the source has only one link (degree one)
        ds[target]+=1 # add one to the target link degree
    return G


def gnr_graph(n,p,seed=None):
    """Return the GNR (growing network with redirection) digraph with n nodes
    and redirection probability p.

    The graph is built by adding nodes one at a time with a link
    to one previously added node.  The previous target node is chosen
    uniformly at random.  With probabiliy p the link is instead "redirected"
    to the successor node of the target.  The graph is always a (directed)
    tree.

    Example:

    >>> D=gnr_graph(10,0.5)  # the GNR graph
    >>> G=D.to_undirected()  # the undirected version

    Reference::

      @article{krapivsky-2001-organization,
      title   = {Organization of Growing Random Networks},
      author  = {P. L. Krapivsky and S. Redner},
      journal = {Phys. Rev. E},
      volume  = {63},
      pages   = {066123},
      year    = {2001},
      }

    """
    G=empty_graph(1,create_using=DiGraph())
    G.name="gnr_graph(%s,%s)"%(n,p)

    if not seed is None:
        random.seed(seed)

    if n==1:
        return G

    for source in range(1,n):
        target=random.randrange(0,source)
        if random.random() < p and target !=0:
            target=G.successors(target)[0]
        G.add_edge(source,target)

    return G



def gnc_graph(n,seed=None):
    """Return the GNC (growing network with copying) digraph with n nodes.

    The graph is built by adding nodes one at a time with a links
    to one previously added node (chosen uniformly at random)
    and to all of that node's successors.

    Reference::

      @article{krapivsky-2005-network,
      title   = {Network Growth by Copying},
      author  = {P. L. Krapivsky and S. Redner},
      journal = {Phys. Rev. E},
      volume  = {71},
      pages   = {036118},
      year    = {2005},
      }

    """
    G=empty_graph(1,create_using=DiGraph())
    G.name="gnc_graph(%s)"%(n)

    if not seed is None:
        random.seed(seed)

    if n==1:
        return G

    for source in range(1,n):
        target=random.randrange(0,source)
        for succ in G.successors(target):
            G.add_edge(source,succ)
        G.add_edge(source,target)

    return G


def _test_suite():
    import doctest
    suite = doctest.DocFileSuite('tests/generators/directed.txt',
                                 package='networkx')
    return suite


if __name__ == "__main__":
    import os
    import sys
    import unittest
    if sys.version_info[:2] < (2, 4):
        print "Python version 2.4 or later required (%d.%d detected)."\
              %  sys.version_info[:2]
        sys.exit(-1)
    # directory of networkx package (relative to this)
    nxbase=sys.path[0]+os.sep+os.pardir
    sys.path.insert(0,nxbase) # prepend to search path
    unittest.TextTestRunner().run(_test_suite())
    



"""
Layout (positioning) algorithms for graph drawing.
"""
__author__ = """Aric Hagberg (hagberg@lanl.gov)\nDan Schult(dschult@colgate.edu)"""
__date__ = "$Date: 2005-06-15 08:53:26 -0600 (Wed, 15 Jun 2005) $"
__credits__ = """"""
__revision__ = "$Revision: 1033 $"
#    Copyright (C) 2004,2005 by 
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Pieter Swart <swart@lanl.gov>
#    Distributed under the terms of the GNU Lesser General Public License
#    http://www.gnu.org/copyleft/lesser.html
from math import pi,sin,cos,sqrt
import sys
from networkx.utils import uniform_sequence

try:
    import Numeric as N
except:
    try:
        import numpy as N
    except:
        raise ImportError,"numpy and/or Numeric can not be imported."    


def circular_layout(G, dim=2):
    """
    Circular layout.

    Crude version that doesn't try to minimize edge crossings.

    """
    vpos={}
    r=1.0  # radius of circle
    t=0
    dt=2.0*pi/G.order()
    # should test for dim < 2 here
    for v in G:
        p=dim*[0.0]
        p[0]=r*cos(t)
        p[1]=r*sin(t)
        vpos[v]=N.array(p)
        t=t+dt
    return vpos        

def shell_layout(G, nlist=None, dim=2 ):
    """
    Shell layout.
    Crude version that doesn't try to minimize edge crossings.

    nlist is an optional list of lists of nodes to be drawn
    at each shell level.  Only one shell with all nodes will
    be drawn if not specified.

    """
    if nlist==None:
        nlist=[G.nodes()] # draw the whole graph in one shell

    nshells=len(nlist)
    vpos={}
    if len(nlist[0])==1:
        initial_radius=0.0 # single node at center
    else:
        initial_radius=1.0

    for s in range(nshells):
        r=float(s+initial_radius)  # radius of circle
        t=0
        slist=[n for n in nlist[s] if G.has_node(n)]
        dt=2.0*pi/len(slist)
        for n in slist:
            p=dim*[0.0]
            p[0]=r*cos(t)
            p[1]=r*sin(t)
            vpos[n]=N.array(p)
            t=t+dt
    return vpos        

def random_layout(G, dim=2):
    """ Random layout."""
    import random
    vpos={}
    for v in G.nodes():
        vpos[v]=N.array([random.random() for i in range(dim)])
    return vpos        

def spring_layout(G, iterations=50, dim=2, node_pos=None):
    """Spring force model layout"""
    if node_pos==None :  # set the initial positions randomly in 1x1 box
        vpos=random_layout(G, dim=dim) 
    else:
        vpos=node_pos
    if iterations==0:
        return vpos
    if G.order()==0:
        k=1.0
    else:
        k=N.sqrt(1.0/G.order()) # optimal distance between nodes
    disp={}         # displacements

    # initial "temperature" (about .1 of domain area)
    # this is the largest step allowed in the dynamics
    # linearly step down by dt on each iteration so
    # on last iteration it is size dt.
    t=0.1
    dt=0.1/float(iterations+1)
    for i in range(0,iterations):
        for v in G:
            disp[v]=N.zeros(dim)
            for u in G:
                delta=vpos[v]-vpos[u]
                dn=max(sqrt(N.dot(delta,delta)),0.01)
                # repulsive force between all
                deltaf=delta*k**2/dn**2
                disp[v]=disp[v]+deltaf
                # attractive force between neighbors
                if G.has_edge(v,u):
                    deltaf=-delta*dn**2/(k*dn)
                    disp[v]=disp[v]+deltaf

        # update positions
        for v in G:
            l=max(sqrt(N.dot(disp[v],disp[v])),0.01)
            vpos[v]=vpos[v]+ disp[v]*t/l
        t-=dt
    return vpos
                   

def spectral_layout(G, dim=2, vpos=None, iterations=1000, eps=1.e-3):
    """
    Return the position vectors for drawing G using spectral layout.
    """
    # check silly cases
    n=G.order()
    if iterations==0 or n==0:
        return vpos or {}
    # create initial guesses for positions
    if vpos is None:
        # start with random positions
        nodes=G.nodes()
        uhat=[ ]
        for p in range(dim):
            rx=uniform_sequence(n)
            uhat.append( dict( zip(nodes,rx) ) )
    else:
        # use given positions
        uhat=[]
        nodes = vpos.keys()
        for p in range(dim):
            rx=[(v,pos[p]) for (v,pos) in vpos.iteritems()]   
            uhat.append( dict(rx) )
    # Find lowest eigenvectors
    vhat=graph_low_ev_pi(uhat,G,eps,iterations)
    # form position dict
    vpos={}
    for n in nodes:
        poslist=[ vhat[p].get(n,0) for p in range(dim) ]
        vpos[n]=N.array( poslist )
    return vpos

def graph_low_ev_pi(uhat,G,eps=1.e-3,iterations=10000):
    """ 
    Power Iteration method to find smallest eigenvectors of Laplacian(G).
    Note: constant eigenvector has eigenvalue=0 but is not included
    in the count of smallest eigenvalues.

    uhat -- list of p initial guesses (dicts) for the p eigenvectors.
    G -- The Graph from which Laplacian is calculated.
    eps -- tolerance for norm of change in eigenvalue estimate.
    iterations -- maximum number of iterations to use.
    """
    # set up element value for constant eigenvector (squared)
    constant=1.0/G.order()
    # set up data for faster iteration
    gg_data=_gershgorin_setup(G)
    # 
    v=[]  # setup v to hold eigenvectors
    p=len(uhat)   # number of guesses is number of vectors to return 
    for i in xrange(p):
        vv=uhat[i]  # get initial guess
        ##normalize(vv)
        norm=sqrt(N.sum([vals**2 for vals in vv.itervalues()]))
        nn=1.0/norm
        for k in vv:
            vv[k] *= nn
        ##
        v.append(vv)

        difference=eps+1
        iii=0
#        sys.stderr.write("drawing")
        while difference>eps:
            vhat=v[i]
            ## Orthogonal to constant vector
            dotprod=constant*sum(vhat.itervalues())
            for (k,val) in vhat.iteritems():
                vhat[k]-=dotprod
            ## Orthogonal to other already found eigenvectors
            for j in range(i):  
                ##  vhat= vhat- <vhat,v[j]> * v[j]
                s_d=sum([vhat.get(k,0)*val for (k,val) in v[j].iteritems()])
                for (k,value) in v[j].iteritems():
                    vhat[k]=vhat.get(k,0)-value*s_d
                ##
            vv=_graph_gershgorin_dot_v(gg_data,vhat)
            ##normalize(vv)     vv=vv/|vv|
            norm=sqrt(sum([vals**2 for vals in vv.itervalues()]))
            if norm==0:
                raise networkx.NetworkXError,"Eigenvector with zero eigenvalue given as input."
            nn=1.0/norm
            for k in vv:
                vv[k] *= nn
            ##
            v[i]=vv
            ##difference = |vhat-vv|
            result=vhat.copy()
            for (k,value) in vv.iteritems():
                result[k]=result.get(k,0)-value
            difference=sqrt(sum([vals**2 for vals in result.itervalues()]))
            #print "difference on iteration %s is %s"%(iii,difference)
            ##
            iii += 1
            if iii==iterations: 
                #print "Maximum iterations achieved."
                break
                raise NetworkXError, "Maximum number of iterations exceeded." 
#            if iii%20==0: sys.stderr.write(".")
    return v 
    
def _gershgorin_setup(G):
    """
    Return a list of matrix properties to be used 
    to iteratively multiply B*v where v is a vector
    and B=g*I-L and g is the Gershgorin estimate of 
    the largest eigenvalue of L=Laplacian(G).

    Used as input to graph_gershgorin_dot_v()
    """
    diag=G.degree(with_labels=True)
    adj={}
    g=max(diag.itervalues())
    for (n,degree) in diag.iteritems():
        diag[n]=g-degree
        adj[n]=G.neighbors(n)
    return [diag,adj]

def _graph_gershgorin_dot_v(gg_data,v):
    """
    Returns B*v where B=g*I-L and g is the Gershgorin estimate of the 
    largest eigenvalue of L.  (g=max( deg(n) + sum_u(\|w_(n,u)\|)

    We use this to iterate and find the smallest eigenvectors of L.
    """
    (diag,adj)=gg_data
    r={}
    for (n,nbrs) in adj.iteritems():
        rn=sum([ v.get(u,0) for u in nbrs ])
        #rn=sum([ v.get(u,0)*w for (u,w) in nbrs.iteritems() ])#weighted adj matrix
        r[n]=rn+v.get(n,0)*diag[n]
    return r

def _test_suite():
    import doctest
    suite = doctest.DocFileSuite('tests/drawing/layout.txt',package='networkx')
    return suite

if __name__ == "__main__":
    import os
    import sys
    import unittest
    if sys.version_info[:2] < (2, 4):
        print "Python version 2.4 or later required for tests (%d.%d detected)." %  sys.version_info[:2]
        sys.exit(-1)
    # directory of networkx package (relative to this)
    nxbase=sys.path[0]+os.sep+os.pardir
    sys.path.insert(0,nxbase) # prepend to search path
    unittest.TextTestRunner().run(_test_suite())
    



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Setup script for networkx

"""

from glob import glob
import os
import sys
if os.path.exists('MANIFEST'): os.remove('MANIFEST')

from distutils.core import setup

if sys.argv[-1] == 'setup.py':
    print "To install, run 'python setup.py install'"
    print

if sys.version_info[:2] < (2, 3):
    print "NX requires Python version 2.3 or later (%d.%d detected)." % \
          sys.version_info[:2]
    sys.exit(-1)

execfile(os.path.join('networkx','release.py'))

packages=["networkx",
          "networkx.generators",
          "networkx.drawing",
          "networkx.readwrite",
          "networkx.tests",
          "networkx.tests.generators",
          "networkx.tests.drawing",
          "networkx.tests.readwrite",
          ]

docdirbase  = 'share/doc/networkx-%s' % version
data = [(docdirbase, glob("doc/*.txt")),
        (os.path.join(docdirbase, 'examples'),glob("doc/examples/*.py")),
        (os.path.join(docdirbase, 'examples'),glob("doc/examples/*.dat")),
        (os.path.join(docdirbase, 'examples'),glob("doc/examples/*.edges")),
        (os.path.join(docdirbase, 'data'),glob("doc/data/*ls")),
        ]

package_data     = {'': ['*.txt'],} 


if __name__ == "__main__":

    setup(name             = name,
      version          = version,
      author           = authors['Hagberg'][0],
      author_email     = authors['Hagberg'][1],
      description      = description,
      keywords         = keywords,
      long_description = long_description,
      license          = license,
      platforms        = platforms,
      url              = url,      
      download_url     = download_url,
      packages         = packages,
      data_files       = data,
      package_data     = package_data, 
      classifiers      = classifiers,
      )




"""
Interface to pygraphviz AGraph class.

Usage 

 >>> from networkx import *
 >>> G=complete_graph(5)
 >>> A=to_agraph(G)
 >>> H=from_agraph(A)

"""
__author__ = """Aric Hagberg (hagberg@lanl.gov)"""
#    Copyright (C) 2004-2006 by 
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Pieter Swart <swart@lanl.gov>
#    Distributed under the terms of the GNU Lesser General Public License
#    http://www.gnu.org/copyleft/lesser.html
import os
import sys
from networkx.utils import _get_fh
try:
    import pygraphviz
except ImportError:
    raise

def from_agraph(A,create_using=None):
    """Return a NetworkX XGraph or XDiGraph from a pygraphviz graph.

    >>> X=from_agraph(A)

    The XGraph X will have a dictionary X.graph_attr containing
    the default graphviz attributes for graphs, nodes and edges.

    Default node attributes will be in the dictionary X.node_attr
    which is keyed by node.

    Edge attributes will be returned as edge data in the graph X.

    If you want a Graph with no attributes attached instead of an XGraph
    with attributes use

    >>> G=Graph(X)

    """
    import networkx
    if A.is_strict():
        multiedges=False
        selfloops=False
    else:
        multiedges=True
        selfloops=True
        
    if create_using is None:        
        if A.is_undirected():
            create_using=networkx.XGraph(multiedges=multiedges,
                                         selfloops=selfloops)
        else:
            create_using=networkx.XDiGraph(multiedges=multiedges,
                                           selfloops=selfloops)

    # assign defaults        
    N=networkx.empty_graph(0,create_using)
    N.name=str(A)
    node_attr={}
    # add nodes, attributes to N.node_attr
    for n in A.nodes():
        N.add_node(n)
        node_attr[n]=n.attr

    # add edges, attributes attached to edge
    for e in A.edges():
        if len(e)==2:
            u,v=e
        else:
            u,v,k=e
        if hasattr(N,'allow_multiedges')==True: # XGraph
            N.add_edge(u,v,e.attr)
        else: # Graph
            N.add_edge(u,v)
        
    # add default attributes for graph, nodes, and edges       
    # hang them on N.graph_attr
    if hasattr(N,'allow_multiedges')==True: # XGraph
        N.graph_attr={}
        N.graph_attr['graph']=A.graph_attr
        N.graph_attr['node']=A.node_attr
        N.graph_attr['edge']=A.edge_attr
        N.node_attr=node_attr

    return N        

def to_agraph(N, graph_attr=None, node_attr=None, edge_attr=None,
              strict=True):
    """Return a pygraphviz graph from a NetworkX graph N.

    If N is a Graph or DiGraph, graphviz attributes can
    be supplied through the arguments

    graph_attr:  dictionary with default attributes for graph, nodes, and edges
                 keyed by 'graph', 'node', and 'edge' to attribute dictionaries

    node_attr: dictionary keyed by node to node attribute dictionary

    edge_attr: dictionary keyed by edge tuple to edge attribute dictionary

    If N is an XGraph or XDiGraph an attempt will be made first
    to copy properties attached to the graph (see from_agraph)
    and then updated with the calling arguments if any.

    """
    directed=N.is_directed()
    if hasattr(N,'allow_multiedges'):
        if N.multiedges:
            strict=False
    if hasattr(N,'allow_selfloops'):
        if N.selfloops:
            strict=False
    A=pygraphviz.AGraph(name=N.name,strict=strict,directed=directed)

    # default graph attributes            
    try:             
        A.graph_attr.update(N.graph_attr['graph'])
    except:
        pass
    try:
        A.graph_attr.update(graph_attr['graph'])
    except:
        pass
    # default node attributes            
    try:        
        A.node_attr.update(N.graph_attr['node'])
    except:
        pass
    try:
        A.node_attr.update(graph_attr['node'])
    except:
        pass
    # default edge attributes            
    try:        
        A.edge_attr.update(N.graph_attr['edge'])
    except:
        pass
    try:
        A.edge_attr.update(graph_attr['edge'])
    except:
        pass

    # add nodes
    for n in N.nodes_iter():
        A.add_node(n)
        node=pygraphviz.Node(A,n)
        # try node attributes attached to graph
        try:
            if n in N.node_attr:
                node.attr.update(N.node_attr[n])
        except:
            pass
        # update with attributes from calling parameters
        try:
            if n in node_attr:
                node.attr.update(node_attr[n])
        except:
            pass

    # loop over edges
    for e in N.edges_iter():
        if len(e)==2:
            (u,v)=e
            data=None
        else:
            (u,v,data)=e

        if data is None: # no data, just add edge
            A.add_edge(u,v)
            edge=pygraphviz.Edge(A,u,v)
        else: # could have list of edge data or single edge data
            try:
                N.allow_multiedges()==True
                dlist=N.get_edge(u,v)
            except:
                dlist=[N.get_edge(u,v)]
            for d in dlist:
                A.add_edge(u,v)
                edge=pygraphviz.Edge(A,u,v)
                if hasattr(d,"__getitem__"):
                    edge.attr.update(d)
        # update from calling argument
        try:  # e might not be hashable
            if (u,v) in edge_attr:
                edge.attr.update(edge_attr[(u,v)])
        except:
            pass

    return A

def write_dot(G,path):
    """Write NetworkX graph G to Graphviz dot format on path.

    Path can be a string or a file handle.
    """
    A=to_agraph(G)
    A.write(path)
    return

def read_dot(path,create_using=None):
    """Return a NetworkX XGraph or XdiGraph from a dot file on path.

    Path can be a string or a file handle.

    """
    A=pygraphviz.AGraph(file=path)
    return from_agraph(A)


def graphviz_layout(G,prog='neato',root=None, args=''):
    """
    Create layout using graphviz.
    Returns a dictionary of positions keyed by node.

    >>> from networkx import *
    >>> G=petersen_graph()
    >>> pos=graphviz_layout(G)
    >>> pos=graphviz_layout(G,prog='dot')
    
    This is a wrapper for pygraphviz_layout.

    """
    return pygraphviz_layout(G,prog=prog,root=root,args=args)

def pygraphviz_layout(G,prog='neato',root=None, args=''):
    """
    Create layout using pygraphviz and graphviz.
    Returns a dictionary of positions keyed by node.

    >>> from networkx import *
    >>> G=petersen_graph()
    >>> pos=pygraphviz_layout(G)
    >>> pos=pygraphviz_layout(G,prog='dot')
    
    """
    A=to_agraph(G)
    A.layout(prog=prog,args=args)
    node_pos={}
    for n in G.nodes():
        node=pygraphviz.Node(A,n)
        try:
            xx,yy=node.attr["pos"].split(',')
            node_pos[n]=(float(xx),float(yy))
        except:
            print "no position for node",n
            node_pos[n]=(0.0,0.0)
    return node_pos


def _test_suite():
    import doctest
    suite = doctest.DocFileSuite('tests/drawing/nx_agraph.txt',package='networkx')
    return suite




if __name__ == "__main__":
    import os
    import sys
    import unittest
    if sys.version_info[:2] < (2, 4):
        print "Python version 2.4 or later required for tests (%d.%d detected)." %  sys.version_info[:2]
        sys.exit(-1)
    # directory of networkx package (relative to this)
    nxbase=sys.path[0]+os.sep+os.pardir
    sys.path.insert(0,nxbase) # prepend to search path
    unittest.TextTestRunner().run(_test_suite())




"""
Read and write NetworkX graphs in YAML format.
See http://www.yaml.org for documentation.

"""
__author__ = """Aric Hagberg (hagberg@lanl.gov)"""
__date__ = """"""
__credits__ = """"""
__revision__ = "$$"
#    Copyright (C) 2004-2007 by 
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Pieter Swart <swart@lanl.gov>
#    Distributed under the terms of the GNU Lesser General Public License
#    http://www.gnu.org/copyleft/lesser.html

import cPickle 
import codecs
import locale
import string
import sys
import time

from networkx.utils import is_string_like, _get_fh
import networkx

def write_yaml(G, path, default_flow_style=False, **kwds):
    """Write graph G in YAML text format to path. 

    See http://www.yaml.org

    """
    try:
        import yaml
    except ImportError:
        raise ImportError, \
              "Import Error: not able to import yaml: http://www.yaml.org "
    fh=_get_fh(path,mode='w')        
    yaml.dump(G,fh,default_flow_style=default_flow_style,**kwds)
    

def read_yaml(path):
    """Read graph from YAML format from path.

    See http://www.yaml.org

    """
    try:
        import yaml
    except ImportError:
        raise ImportError, \
              "Import Error: not able to import yaml: http://www.yaml.org "

    fh=_get_fh(path,mode='r')        
    return yaml.load(fh)



def _test_suite():
    import doctest
    import yaml
    suite = doctest.DocFileSuite('tests/readwrite/nx_yaml.txt',package='networkx')
    return suite

if __name__ == "__main__":
    import os 
    import sys
    import unittest
    if sys.version_info[:2] < (2, 4):
        print "Python version 2.4 or later required for tests (%d.%d detected)." %  sys.version_info[:2]
        sys.exit(-1)
    # directory of networkx package (relative to this)
    nxbase=sys.path[0]+os.sep+os.pardir
    sys.path.insert(0,nxbase) # prepend to search path
    unittest.TextTestRunner().run(_test_suite())
    



"""Release data for NetworkX."""

#    Copyright (C) 2004-2008 by 
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Pieter Swart <swart@lanl.gov>
#    Distributed under the terms of the GNU Lesser General Public License
#    http://www.gnu.org/copyleft/lesser.html

__author__ = """Aric Hagberg (hagberg@lanl.gov)\nPieter Swart (swart@lanl.gov)\nDan Schult (dschult@colgate.edu)"""

name = 'networkx'
version = '0.36'

description = "Python package for creating and manipulating graphs and networks"

long_description = \
"""
NetworkX is a Python package for the creation, manipulation, and
study of the structure, dynamics, and functions of complex networks.  

"""
license = 'LGPL'
authors = {'Hagberg' : ('Aric Hagberg','hagberg@lanl.gov'),
           'Schult' : ('Dan Schult','dschult@colgate.edu'),
           'Swart' : ('Pieter Swart','swart@lanl.gov')
           }
url = 'http://networkx.lanl.gov/'
download_url="http://networkx.lanl.gov/download"
platforms = ['Linux','Mac OSX','Windows XP/2000/NT']
keywords = ['Networks', 'Graph Theory', 'Mathematics', 'network', 'graph', 'discrete mathematics', 'math']
classifiers = [
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'Topic :: Scientific/Engineering :: Information Analysis',
        'Topic :: Scientific/Engineering :: Mathematics',
        'Topic :: Scientific/Engineering :: Physics',
        ]

# Get date dynamically
import time
date = time.asctime()
del time




#!/usr/bin/env python
"""
Centrality measures of Krackhardt social network.
"""
__author__ = """Aric Hagberg (hagberg@lanl.gov)"""
__date__ = "$Date: 2005-05-12 14:33:11 -0600 (Thu, 12 May 2005) $"
__credits__ = """"""
__revision__ = "$Revision: 998 $"
#    Copyright (C) 2004 by 
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Pieter Swart <swart@lanl.gov>
#    Distributed under the terms of the GNU Lesser General Public License
#    http://www.gnu.org/copyleft/lesser.html

from networkx import *

G=krackhardt_kite_graph()

print "Betweenness"
b=betweenness_centrality(G)
for v in G.nodes():
    print "%0.2d %5.3f"%(v,b[v])

print "Degree centrality"
d=degree_centrality(G)
for v in G.nodes():
    print "%0.2d %5.3f"%(v,d[v])

print  "Closeness centrality"
c=closeness_centrality(G)
for v in G.nodes():
    print "%0.2d %5.3f"%(v,c[v])



"""
NetworkX
========

   NetworkX (NX) is a Python package for the creation, manipulation, and
   study of the structure, dynamics, and functions of complex networks.  

   https://networkx.lanl.gov/

Using 
-----

   Just write in Python

   >>> import networkx as NX
   >>> G=NX.Graph()
   >>> G.add_edge(1,2)
   >>> G.add_node("spam")
   >>> print G.nodes()
   [1, 2, 'spam']
   >>> print G.edges()
   [(1, 2)]
"""
#    Copyright (C) 2004-2006 by 
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Pieter Swart <swart@lanl.gov>
#    Distributed under the terms of the GNU Lesser General Public License
#    http://www.gnu.org/copyleft/lesser.html
#
# Add platform dependent shared library path to sys.path
#
import sys
if sys.version_info[:2] < (2, 3):
    print "Python version 2.3 or later is required for NetworkX (%d.%d detected)." %  sys.version_info[:2]
    sys.exit(-1)
del sys
# Release data
import release 
__version__  = release.version
__date__     = release.date
__author__   = '%s <%s>\n%s <%s>\n%s <%s>' % \
              ( release.authors['Hagberg'] + release.authors['Schult'] + \
                release.authors['Swart'] )
__license__  = release.license

import info
__doc__+=info.__doc__

#
# NetworkX package modules
#
from exception import  NetworkXException, NetworkXError
from graph import Graph
from digraph import DiGraph
from function import  nodes, edges, degree, degree_histogram, neighbors,\
                 number_of_nodes, number_of_edges, density,\
                 nodes_iter, edges_iter, is_directed
from xgraph import XGraph
from xdigraph import XDiGraph
from distance import eccentricity, diameter, radius, periphery, center
from path import  \
     shortest_path, shortest_path_length,bidirectional_shortest_path,\
     single_source_shortest_path, single_source_shortest_path_length,\
     all_pairs_shortest_path, all_pairs_shortest_path_length,\
     dijkstra_path, dijkstra_path_length, bidirectional_dijkstra,\
     single_source_dijkstra_path, single_source_dijkstra_path_length,\
     single_source_dijkstra,\
     dijkstra_predecessor_and_distance, predecessor, floyd_warshall
from dag import \
     topological_sort, topological_sort_recursive,\
     is_directed_acyclic_graph
from component import \
     number_connected_components, connected_components,\
     is_connected, connected_component_subgraphs,\
     node_connected_component,\
     number_strongly_connected_components, strongly_connected_components,\
     is_strongly_connected, strongly_connected_component_subgraphs,\
     strongly_connected_components_recursive,\
     kosaraju_strongly_connected_components

from search import dfs_preorder, dfs_postorder, dfs_predecessor,\
            dfs_successor, dfs_tree
from cluster import triangles, average_clustering, clustering, transitivity
from operators import union, cartesian_product, compose, complement,\
                      disjoint_union, create_empty_copy,\
                      subgraph, convert_to_undirected, convert_to_directed,\
                      convert_node_labels_to_integers, relabel_nodes
from centrality import betweenness_centrality, \
                       betweenness_centrality_source, \
                       load_centrality,\
                       newman_betweenness_centrality,\
                       brandes_betweenness_centrality,\
                       edge_betweenness,\
                       edge_load,\
                       degree_centrality, \
                       closeness_centrality
from hybrid import kl_connected_subgraph, is_kl_connected
from cliques import find_cliques, make_max_clique_graph,\
                    make_clique_bipartite,graph_clique_number,\
                    graph_number_of_cliques,node_clique_number,\
                    number_of_cliques,cliques_containing_node
from isomorph import is_isomorphic
                    
# need numpy for spectrum
try:
    from spectrum import \
         adj_matrix, laplacian, generalized_laplacian,\
         laplacian_spectrum, adjacency_spectrum
except ImportError:
    pass

from utils import is_string_like, iterable,\
                  pareto_sequence, powerlaw_sequence, uniform_sequence,\
                  cumulative_distribution, discrete_sequence,_get_fh


from convert import from_whatever,\
     from_dict_of_dicts, to_dict_of_dicts,\
     from_dict_of_lists, to_dict_of_lists,\
     from_numpy_matrix, to_numpy_matrix,\
     from_scipy_sparse_matrix, to_scipy_sparse_matrix

# import some useful graphs - we always use these...
from generators.classic import *
from generators.small import *
from generators.random_graphs import *
from generators.degree_seq import *
from generators.directed import *
# drawing
from drawing import *
# read/write graphs in various formats
from readwrite import *

from tests import run as test



"""
Laplacian, adjacency matrix, and spectrum of graphs.

Needs numpy array package: numpy.scipy.org.

"""
__author__ = """Aric Hagberg (hagberg@lanl.gov)\nPieter Swart (swart@lanl.gov)\nDan Schult(dschult@colgate.edu)"""
#    Copyright (C) 2004-2006 by 
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Pieter Swart <swart@lanl.gov>
#    Distributed under the terms of the GNU Lesser General Public License
#    http://www.gnu.org/copyleft/lesser.html

try:
    import numpy as N
except:
    raise ImportError, 'numpy not found'

import networkx

def adj_matrix(G,nodelist=None):
    """Return adjacency matrix of graph as a numpy matrix.

    This just calls networkx.convert.to_numpy_matrix.

    If you want a pure python adjacency matrix represntation try
    networkx.convert.to_dict_of_dicts with weighted=False,
    which will return a dictionary-of-dictionaries format that
    can be addressed as a sparse matrix.

    """
    return networkx.to_numpy_matrix(G,nodelist=nodelist)


def laplacian(G,nodelist=None):
    """Return standard combinatorial Laplacian of G as a numpy matrix.

    Return the matrix L = D - A, where

       D is the diagonal matrix in which the i'th entry is the degree of node i
       A is the adjacency matrix.

    """
    # this isn't the most efficient way to do this...
    n=G.order()
    I=N.identity(n)
    A=N.asarray(networkx.to_numpy_matrix(G,nodelist=nodelist))
    D=I*N.sum(A,axis=1)
    L=D-A
    return L


def normalized_laplacian(G,nodelist=None):
    """Return normalized Laplacian of G as a numpy matrix.

    See Spectral Graph Theory by Fan Chung-Graham.
    CBMS Regional Conference Series in Mathematics, Number 92,
    1997.

    """
    # FIXME: this isn't the most efficient way to do this...
    n=G.order()
    I=N.identity(n)
    A=N.asarray(networkx.to_numpy_matrix(G,nodelist=nodelist))
    d=N.sum(A,axis=1)
    L=I*d-A
    osd=N.zeros(len(d))
    for i in range(len(d)):
        if d[i]>0: osd[i]=N.sqrt(1.0/d[i])
    T=I*osd
    L=N.dot(T,N.dot(L,T))
    return L

def laplacian_spectrum(G):
    """Return eigenvalues of the Laplacian of G""" 
    return N.linalg.eigvals(laplacian(G))

def adjacency_spectrum(G):
    """Return eigenvalues of the adjacency matrix of G""" 
    return N.linalg.eigvals(adj_matrix(G))


combinatorial_laplacian=laplacian
generalized_laplacian=normalized_laplacian


def _test_suite():
    import doctest
    suite = doctest.DocFileSuite('tests/spectrum.txt',package='networkx')
    return suite


if __name__ == "__main__":
    import os
    import sys
    import unittest
    if sys.version_info[:2] < (2, 4):
        print "Python version 2.4 or later required for tests (%d.%d detected)." %  sys.version_info[:2]
        sys.exit(-1)
    # directory of networkx package (relative to this)
    nxbase=sys.path[0]+os.sep+os.pardir
    sys.path.insert(0,nxbase) # prepend to search path
    unittest.TextTestRunner().run(_test_suite())
    



#!/usr/bin/env python
"""
Load football network in GML format and compute some network statistcs.

Shows how to download GML graph in a zipped file, unpack it, and load
into a NetworkX graph.

Requires Internet connection to download the URL
http://www-personal.umich.edu/~mejn/netdata/football.zip

"""
__author__ = """Aric Hagberg (hagberg@lanl.gov)"""
#    Copyright (C) 2007 by 
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Pieter Swart <swart@lanl.gov>
#    Distributed under the terms of the GNU Lesser General Public License
#    http://www.gnu.org/copyleft/lesser.html

try:
    import pyparsing
except ImportError:
    raise "pyparsing not found: install http://pyparsing.wikispaces.com/"
    
from networkx import *

url="http://www-personal.umich.edu/~mejn/netdata/football.zip"

import urllib                                       
import StringIO
import zipfile

sock = urllib.urlopen(url)  # open URL
s=StringIO.StringIO(sock.read()) # read into StringIO "file"
sock.close()                                        

zf = zipfile.ZipFile(s) # zipfile object
txt=zf.read('football.txt') # read info file
gml=zf.read('football.gml') # read gml data
# throw away bogus first line with # from mejn files
gml=gml.split('\n')[1:]
G=parse_gml(gml) # parse gml data

print txt
# print degree for each team - number of games
for n,d in G.degree(with_labels=True).items():
    print n,d
